;;; -*- mode: TI-Asm; ti-asm-listing-file: "units.lst" -*-
;;;
;;;	Unit Operations Flash App for TI-83+
;;; 
;;;	Copyright 2003 Benjamin Moody
;;;
;;;  This program is free software; you can redistribute
;;;  and/or modify it under the terms of the GNU General
;;;  Public License as published by the Free Software
;;;  Foundation; either version 2 of the License, or (at
;;;  your option) any later version.
;;;
;;;  This program is distributed in the hope that it will be
;;;  useful, but WITHOUT ANY WARRANTY; without even the
;;;  implied warranty of MERCHANTABILITY or FITNESS FOR A
;;;  PARTICULAR PURPOSE. See the GNU General Public License
;;;  for more details.
;;;
;;;  You should have received a copy of the GNU General
;;;  Public License along with this program; if not, write
;;;  to the Free Software Foundation, Inc., 59 Temple Place
;;;  - Suite 330, Boston, MA 02111 USA.
;;;

;;; hooks.asm: the basic hook code and the menu system

CxM_HomeHook:
	.db $83
	or a
	jr nz,CxM_HomeHook_NotDisp
	call SetOP1Units_NO_STRING
	jr nz,CxM_HomeHook_NotUnit
	ld hl,NoUnits+2
	ld de,OP2+2
	ld b,7
	call CompStrsN
	jr nz,CxM_Home_NotReal
	call RealOP1
	jr CxM_HomeHook_NotUnit
CxM_Home_NotReal:
	call ConvertToString
	call FillString

	;; Curiously, although the hook is given an opportunity to
	;; inspect and modify the result to be displayed, the actual
	;; result of the calculation has already both been stored to
	;; Ans, and has been formatted and written to fmtString.  So
	;; if we want our string to scroll, we need to format it
	;; ourselves.  (Strangely, it seems that if Ans is a string
	;; variable, TIOS will also sometimes re-format what the hook
	;; returns.)  This way, Ans is still the complex variable and
	;; the correct string is displayed on the homescreen.

	B_CALL _ChkFindSym
	jp c,ErrUndefined	; that's bad
	ex de,hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	inc hl
	ld de,fmtString
CxM_Home_WriteLoop:
	ld a,b
	or c
	jr z,CxM_Home_WriteDone
	push hl
	push bc
	push de
	B_CALL _Get_Tok_Strng
	pop de
	ld hl,OP3
	ld c,a
	ld b,0
	ldir
	pop bc
	pop hl
	dec bc
	ld a,(hl)
	inc hl
	B_CALL _IsA2ByteTok
	jr nz,CxM_Home_WriteLoop
	inc hl
	dec bc
	jr CxM_Home_WriteLoop
CxM_Home_WriteDone:
	xor a
	ld (de),a
	ret

CxM_HomeHook_NotUnit:
	xor a
CxM_HomeHook_NotDisp:
	cp a
	push af
	push bc
	call LoadAppVar_RKH
	jr c,CxM_HomeHook_SkipSecond
	ld hl,OFF_HOMEHOOK
	add hl,de
	ld de,cxM_HomeHookPtr
	call Mov3
	ld a,(cxM_HomeHookPtr+2)
	ld b,a
	ld hl,(cxM_HomeHookPtr)
	B_CALL _LoadCIndPaged
	ld a,$83
	sub c
	jr nz,CxM_HomeHook_Restore
	ld e,a			; Easiest way to call hook: E=0
	pop bc
	pop af
	B_CALL cxM_HomeHookPtr+$4000
	;; (I know this works on 1.12 and 1.16, the former being installed in
	;; my Tilem at the moment, and the latter on my calculators. If it
	;; doesn't work on your calculator, let me know.)
	push af
	push bc
	call LoadAppVar_RKH
	jr c,CxM_HomeHook_Restore ; err...
	ld hl,OFF_HOMEHOOK
	add hl,de
	ex de,hl
	ld hl,cxM_HomeHookPtr
	call Mov3
CxM_HomeHook_Restore:
	ld hl,CxM_HomeHook
	ld (cxM_HomeHookPtr),hl
	in a,(6)
	ld (cxM_HomeHookPtr+2),a
CxM_HomeHook_SkipSecond:
	pop bc
	pop af
	ret


ParserHook:
	.db $83
	or a
	jp z,ParserHook_Preparse
ParserHook_Normal:
	cp a
	push af
	push bc
	push de
	push hl

	;; We're only concerned with A=1 functions
	dec a
	jr z,ParserHook_Type1
	;; and conversions (which have A=2 and C=3)
	dec a
	jr nz,ParserInvalid
	ld a,c
	cp 3
	jr nz,ParserInvalid
	ld a,b
	cp 4
	jr nc,ParserInvalid
ParserHook_Type1:

	ex de,hl
	ld hl,(FPS)
	push hl

	ld hl,ParserDone
	push hl
	push de
	ld l,b
	ld h,0
	add hl,hl
	ld de,ParserTable
	add hl,de
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl
	jp (hl)

ParserDone:
	pop de
	jr z,ParserFail
	pop hl
	pop de
	pop bc
	pop af
	;; If we are returning a unit number, and if it has dimension zero,
	;; set it to be real to avoid TIOS problems
	call SetOP1Units_NO_STRING
	ret nz
	ld hl,OP2+2
	ld de,NoUnits+2
	ld b,7
	call CompStrsN
	ret nz
	call RealOP1
	or 1
	ret
ParserFail:
	ld (FPS),de
ParserInvalid:
	call LoadAppVar_RKH
	jr c,ParserHook_SkipSecond
SecondParserHook:
	ld hl,OFF_PARSER
	add hl,de
	ld de,parserHookPtr
	call Mov3
	ld a,(parserHookPtr+2)
	or a
	jr z,ParserHook_Restore
	ld b,a
	ld hl,(parserHookPtr)
	B_CALL _LoadCIndPaged
	ld a,$83
	sub c
	jr nz,ParserHook_Restore
	ld e,a
	pop hl
	pop de
	pop bc
	pop af
	B_CALL parserHookPtr+$4000
	push af
	push bc
	push de
	push hl
	call LoadAppVar_RKH
	jr c,ParserHook_Restore
	ld hl,OFF_PARSER
	add hl,de
	ex de,hl
	ld hl,parserHookPtr
	call Mov3
ParserHook_Restore:
	ld hl,ParserHook
	ld (parserHookPtr),hl
	in a,(6)
	ld (parserHookPtr+2),a
ParserHook_SkipSecond:
	pop hl
	pop de
	pop bc
	pop af
	ret

ParserTable:
	;; 00
	.dw pzr,    ToDMS,  ToDec,  ToAbc
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw Recip,  Square, pzr,    Cube
	;; 10
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; 20
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; 30
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; 40
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; 50
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; 60
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    EQ,     LT
	.dw GT,     LE,     GE,     NE
	;; 70
	.dw Add,    Sub,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; 80
	.dw pzr,    pzr,    Mul,    Div
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; 90
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; A0
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; B0
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw Sqrt,   CubeRt, pzr,    pzr
	;; C0
	.dw pzr,    pzr,    Sin,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; D0
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; E0
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	;; F0
	.dw Power,  XRoot,  pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr
	.dw pzr,    pzr,    pzr,    pzr

;;; Bytes of appvar used by rawkeyhook:
;;;  -1 Number of bytes to be copied
;;;  -2 High byte of whence to copy
;;;  -3 Low byte of whence to copy
;;;  -4 1 + last prefix entered, bit 7 set if we're currently entering it


; RawkeyHook_Ln:
; 	call LoadAppVar_RKH
; 	jr c,RawkeyHook_Ln_skip
; 	dec de
; 	ld a,(de)		; Count of bytes left to copy
; 	or a
; 	jr z,RawkeyHook_Ln_skip
; 	inc a
; 	jr z,RawkeyHook_Ln_last
; 	ld a,skLog
; 	ld (kbdScanCode),a
; 	set kbdSCR,(iy+kbdFlags)
; 	jr RawkeyHook_Ln_done
; RawkeyHook_Ln_last:
; 	xor a
; 	ld (de),a
; RawkeyHook_Ln_done:
; 	ld a,kQuote
; 	or a
; 	ret
; RawkeyHook_Ln_skip:
; 	or kLn
; 	ret

RawkeyHook_Ln:
	call LoadAppVar_RKH
	jr c,RawkeyHook_Ln_skip
	dec de
	ld a,(de)
	or a
	jr z,RawkeyHook_Ln_skip
	xor a
	ld (de),a
	ld a,kQuote
	or a
	ret
RawkeyHook_Ln_skip:
	ld a,kLn
	or a
	ret

; RawkeyHook_Log:
; 	call LoadAppVar_RKH
; 	jr c,RawkeyHook_Log_skip
; 	dec de
; 	ld a,(de)
; 	or a
; 	jr z,RawkeyHook_Log_skip
; 	ld b,a
; 	ex de,hl
; 	dec hl
; 	ld d,(hl)
; 	dec hl
; 	ld e,(hl)
; 	ld a,(de)
; 	cp $bb
; 	scf
; 	jr nz,Rawkeyhook_Log_onebyte
; 	inc de
; 	dec b
; 	ld a,(de)
; 	or a
; Rawkeyhook_Log_onebyte:
; 	ld c,a
; 	inc de
; 	ld (hl),e
; 	inc hl
; 	ld (hl),d
; 	inc hl
; 	ld a,skLog
; 	dec b
; 	ld (hl),b
; 	jr nz,RawkeyHook_Log_done

; 	;; If a prefix, we don't want to add the closing quote
; 	dec hl
; 	dec hl
; 	dec hl
; 	bit 7,(hl)
; 	jr nz,RawkeyHook_Log_enterpre
; 	inc hl
; 	inc hl
; 	inc hl

; 	ld (hl),$ff
; 	ld a,skLn
; RawkeyHook_Log_done:
; 	ld (kbdScanCode),a
; 	set kbdSCR,(iy+kbdFlags)
; 	ld b,0
; 	ld hl,TokToKeyTable+1
; 	jr c,Rawkeyhook_Log_oneb_2
; 	ld hl,TokToKeyTable2+1
; Rawkeyhook_Log_oneb_2:
; 	add hl,bc
; 	add hl,bc
; 	ld a,(hl)
; 	ld (keyExtend),a
; 	dec hl
; 	ld a,(hl)
; 	or a
; 	ret
; RawkeyHook_Log_skip:
; 	or kLog
; 	ret
; RawkeyHook_Log_enterpre:
; 	res 7,(hl)
; 	ld a,skStat
; 	jr RawkeyHook_Log_done

RawkeyHook:
	.db $83

	;; This is rather important
	set lwrCaseActive,(iy+appLwrCaseFlag)

	ld b,a
	or 1
	ld a,b

	push af
	call LoadAppVar_RKH
	jr c,RawkeyHook_SkipSecond
	ld hl,OFF_RAWKEY
	add hl,de
	ld de,rawKeyHookPtr
	call Mov3
	ld a,(rawKeyHookPtr+2)
	ld b,a
	ld hl,(rawKeyHookPtr)
	B_CALL _LoadCIndPaged
	ld a,$83
	sub c
	jr nz,RawkeyHook_Restore
	ld e,a
	pop af
	B_CALL rawKeyHookPtr+$4000
	push af
	call LoadAppVar_RKH
	jr c,RawkeyHook_Restore
	ld hl,OFF_RAWKEY
	add hl,de
	ex de,hl
	ld hl,rawKeyHookPtr
	call Mov3
RawkeyHook_Restore:
	ld hl,RawkeyHook
	ld (rawKeyHookPtr),hl
	in a,(6)
	ld (rawKeyHookPtr+2),a
RawkeyHook_SkipSecond:
	pop af
	ret z

	;; Ln, converting to Quote, is used to jump-start the recaller
	cp kLn
	jp z,RawkeyHook_Ln

	;; Press Stat twice to get our menu.
	cp kStat
	ret nz

	ld a,(MenuCurrent)
	cp 7			; normal stat menu
	jr z,RawkeyHook_StatMenuOK
	cp 46			; program editor: why is this different?
	jr z,RawkeyHook_StatMenuOK

	;; Stat menu is unavailable during program input, for reasons unknown
	ld a,(cxCurApp)
	cp cxPrgmInput
	jr nz,RawkeyHook_NotPrgmInput

	;; In program input, use VARS STAT instead
	ld a,(MenuCurrent)
	cp $0b
	jr z,RawkeyHook_StatMenuOK
RawkeyHook_NotPrgmInput:

	ld a,kStat
	or a
	ret

; 	;; Status is 01-7F for a previous prefix, 81-FF if we are currently
; 	;; typing one
; 	call LoadAppVar_RKH
; 	jr c,RawKeyHook_NoAV
; 	ld hl,-4
; 	add hl,de
; 	bit 7,(hl)
; 	jr z,RawkeyHook_Prefixed
; 	ld (hl),0
; RawkeyHook_Prefixed:

; 	;; If we're typing a prefix, we want to go immediately to the unit menu
; 	ld a,(hl)
; 	or a
; 	ld a,kStat
; 	ret z
; 	ld a,skStat
; 	ld (kbdScanCode),a
; 	set kbdSCR,(iy+kbdFlags)
; RawKeyHook_NoAV:
; 	ld a,kStat
; 	or a
; 	ret

RawkeyHook_StatMenuOK:

	;; Save a bit of the system state while we display the unit menu

	res rawKeyHookActive,(iy+rawKeyHookFlag)
	ld hl,(curRow)
	push hl
	ld a,(flags+appFlags)
	push af
	xor a
	ld (flags+appFlags),a
	res indicRun,(iy+indicFlags)

	;; Do we have the avar? If not, it's too late now.

	call LoadAppVar_RKH
	jr nc,UnitMenuAppVarOK
	B_CALL _ClrLCDFull
	ld hl,0
	ld (curRow),hl
	ld hl,UnitMenuNoAVStr
	call PutS
	B_CALL _GetKey
	jp UnitMenuExit

UnitMenuAppVarOK:
	xor a
	ld (menuPrefix),a
UnitMenuRestart:
	ld hl,0
	ld (menuX),hl
	ld (menuZ),hl
UnitMenuRedrawAll:
	B_CALL _ClrLCDFull
	ld hl,0
	ld (curRow),hl
	ld a,(menuPrefix)
	ld hl,UnitMenuHeaders
	or a
	jr z,UnitMenuNoCurrentPrefix
	call UnitMenuDrawPrefix
	ld hl,0
	ld (curRow),hl
	ld hl,UnitMenuHeadersPrefixed
UnitMenuNoCurrentPrefix:
	ld a,(menuX)
	ld c,a
	ld b,3
UnitMenuDrawHeaderLoop:
	res textInverse,(iy+textFlags)
	ld a,c
	or a
	jr nz,UnitMenuDrawHeaderNotSel
	ld e,(hl)
	inc hl
	ld d,(hl)
	dec hl
	ld (menuPtr),de		; Save pointer to menu items.
	set textInverse,(iy+textFlags)
UnitMenuDrawHeaderNotSel:
	inc hl
	inc hl
	call PutS
	res textInverse,(iy+textFlags)
	ld a,' '
	B_CALL _PutC
	dec c
	djnz UnitMenuDrawHeaderLoop

UnitMenuRedrawItemsOnly:
	ld hl,1
	ld (curRow),hl

	ld hl,(menuPtr)
	ld b,(hl)		; Total # of items
	inc hl
	ld a,(menuZ)
	ld e,a			; current position
	ld d,0
	add hl,de
	add hl,de
	ld c,e
UnitMenuRedrawItemsLoop:
	push bc
	ld a,c
	cp b
	jp z,UnitMenuRedrawDone
	res textInverse,(iy+textFlags)
	ld a,(menuY)
	cp c
	jr nz,UnitMenuDrawItemNotSel
	set textInverse,(iy+textFlags)
UnitMenuDrawItemNotSel:
	ld a,c
	cp 37
	ld a,' '
	jr nc,UnitMenuDrawItemNoChar
	ld a,(menuSub)
	cp SUB_ALPH
	ld a,' '
	jr nc,UnitMenuDrawItemNoChar
	push hl
	 ld hl,MenuCharTable
	 ld e,c
	 ld d,0
	 add hl,de
	 ld a,(hl)
	pop hl
UnitMenuDrawItemNoChar:
	B_CALL _PutC
	call GetMenuChar
	B_CALL _PutC
	res textInverse,(iy+textFlags)

	call UnitMenuDrawItemText

	pop bc
	inc c
	ld a,(curRow)
	cp 8
	jp c,UnitMenuRedrawItemsLoop
	push bc
UnitMenuRedrawDone:
	pop bc
	ld a,b
	ld (menuBot),a
UnitMenuClearBottomLoop:
	ld a,(curRow)
	cp 8
	jr nc,UnitMenuKeyLoop
	call EraseForceNewLine
	jr UnitMenuClearBottomLoop
UnitMenuUnknownKey:
	ld hl,SpecialKeys
	ld bc,29
	cpir
	jp z,UnitMenuSpecial
UnitMenuNoTopArrow:
UnitMenuKeyLoop:
	call Rawkey_Sub_Getkey
	cp kEnter
	jr z,UnitMenuSel
	cp kClear
	jp z,UnitMenuExitSub
	cp kStat
	jp z,UnitMenuExit
	cp kDown
	jp z,UnitMenuDown
	cp kUp
	jp z,UnitMenuUp
	cp kAlphaDown
	jp z,UnitMenuAlphaDown
	cp kAlphaUp
	jp z,UnitMenuAlphaUp
	cp kLeft
	jp z,UnitMenuLeft
	cp kRight
	jp z,UnitMenuRight
	cp kAdd
	jp z,UnitMenuHelp
	cp k0
	jr c,UnitMenuUnknownKey
	jr z,UnitMenuZeroKey
	cp kCapZ+1
	jr nc,UnitMenuKeyLoop
	cp k9+1
	jr c,UnitMenuNumKey
	add a,k9-kCapA+2
	jr UnitMenuNumKey
UnitMenuZeroKey:
	ld a,k9+1
UnitMenuNumKey:
	ld b,a
	ld a,(menuSub)
	cp SUB_ALPH
	ld a,b
	jp nc,UnitMenuKeyAlph
	sub k1
	ld c,a
	ld a,(menuBot)
	dec a
	cp c
	ld a,c
	jr nc,UnitMenuPick
	jr UnitMenuKeyLoop
UnitMenuSel:
	ld a,(menuY)
	ld c,a
UnitMenuPick:
	ld (menuY),a
	ld b,0
	ld hl,(menuPtr)
	inc hl
	add hl,bc
	add hl,bc
	ld e,(hl)
	inc hl
	ld a,(hl)
	or a
	jp z,UnitMenuSub
	ld d,a
	push de
	call LoadAppVar_RKH
	jp c,UnitMenuMemoryError
	ex de,hl
	pop de

	;; Are we typing a new prefix?
	ld a,(menuX)
	dec a
	jr nz,UnitMenuNotAPrefix
	ld a,(menuPrefix)
	or a
	jr nz,UnitMenuNotAPrefix
	ld a,(menuY)
	add a,$81
 	ld (menuPrefix),a
	jp UnitMenuRestart
UnitMenuNotAPrefix:

	;; Yes, there is a drawback to rcl
	bit editOpen,(iy+editFlags)
	jr z,UnitMenuNotInEdit

	;; Set the flag in the appvar
	dec hl
	ld (hl),1


recallBuffer = tempLoadingOPs+66

;;; There are actually two modes of recall.

;;; "Inverse" mode works the way you would expect it to: it types data
;;; from (rclQueue) up to (rclQueueEnd) into the edit buffer, one
;;; token at a time, displaying them as it goes.  I'm not sure where,
;;; if anywhere, TI uses this mode.  It also doesn't work in the
;;; program editor, or I'm missing some crucial step.  It does have
;;; the advantage that TI's code will take care of worrying about
;;; memory.

;;; "Standard" mode types data directly from the gap, displaying
;;; tokens one by one, from (editCursor) up to (rclQueue).  Therefore
;;; we must copy the unit string into the edit buffer ourselves.

#ifdef INVERSE_RCL
	ld hl,recallBuffer
#else
	ld hl,(editCursor)
	ld (hl),tString
	inc hl
#endif
	
	ex de,hl

	;; Is there a prefix?
	ld a,(menuPrefix)
	or a
	jr z,UnitMenuNotPrefixed
	push hl
	sub $81
	ld c,a
	ld b,0
	ld hl,MenuPrefix+1
	add hl,bc
	add hl,bc
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	ld c,(hl)
	inc hl
#ifndef INVERSE_RCL
	ex de,hl
	push hl
	push bc
	add hl,bc
	ld bc,(editTail)
	sbc hl,bc
	pop bc
	pop hl
	jr nc,UnitMenuMemoryError
	ex de,hl
#endif
	ldir

	pop hl
UnitMenuNotPrefixed:

	ld c,(hl)
	ld b,0
	inc hl
#ifndef INVERSE_RCL
	ex de,hl
	push hl
	push bc
	add hl,bc
	ld bc,(editTail)
	sbc hl,bc
	pop bc
	pop hl
	jr nc,UnitMenuMemoryError
	ex de,hl
#endif
	ldir
	or a
	ex de,hl
	ld (hl),tString
	inc hl

	set 7,(iy+$0e)		; enable rcl

#ifdef INVERSE_RCL
	ld (rclQueueEnd),hl
	ld hl,recallBuffer
	ld (rclQueue),hl
	set 2,(iy+$33)		; inverse rcl
#else
	ld (rclQueue),hl
	res 2,(iy+$33)
#endif

	;; TI always sets insert mode while recalling.  But then
	;; again, older versions of Units would overwrite unless you
	;; had enabled insert mode yourself.  It would be nicer if we
	;; didn't do this; we could then use an arrow key to initiate
	;; recall instead of this ugly Ln key.  Oh well.

	set textInsMode,(iy+textFlags)

	ld a,skLn
	ld (kbdScanCode),a
	set kbdSCR,(iy+kbdFlags)

	res cmdVirgin,(iy+cmdFlags)

UnitMenuNotInEdit:
UnitMenuMemoryError:
	ld a,kClear
	or a
UnitMenuDone:
	;; Reverse everything we did earlier
	pop bc
	ld (iy+appFlags),b
	pop hl
	ld (curRow),hl
	set rawKeyHookActive,(iy+rawKeyHookFlag)
	ret

UnitMenuExit:
	ld a,kStat
UnitMenuSpecial:
	or a
	jr UnitMenuDone
UnitMenuExitSub:
	ld a,(menuSub)
	or a
	ld a,kClear
	jr z,UnitMenuSpecial
	xor a
	ld (menuSub),a
	ld h,a
	ld l,a
	ld (menuY),hl
	jp UnitMenuRedrawAll
UnitMenuUp:
	ld a,(menuY)
	or a
	jr z,UnitMenuUpBot
	ld b,a
	ld a,(menuZ)
	cp b
	jr nz,UnitMenuUpNotTOS
	dec a
	ld (menuZ),a
UnitMenuUpNotTOS:
	ld a,b
	dec a
	ld (menuY),a
	jp UnitMenuRedrawItemsOnly
UnitMenuUpBot:
	ld a,(menuBot)
	dec a
	ld (menuY),a
	sub 6
	ld (menuZ),a
	jp nc,UnitMenuRedrawItemsOnly
	xor a
	ld (menuZ),a
	jp UnitMenuRedrawItemsOnly
UnitMenuDown:
	ld a,(menuY)
	inc a
	ld b,a
	ld a,(menuBot)
	cp b
	jr z,UnitMenuDownTop
	jr c,UnitMenuDownTop
	ld a,(menuZ)
	add a,6
	cp b
	jr nc,UnitMenuDownNotBOS
	sub 5
	ld (menuZ),a
UnitMenuDownNotBOS:
	ld a,b
	ld (menuY),a
	jp UnitMenuRedrawItemsOnly
UnitMenuDownTop:
	ld hl,0
	ld (menuY),hl
	jp UnitMenuRedrawItemsOnly
UnitMenuLeft:
	ld a,(menuSub)
	or a
	jp nz,UnitMenuKeyLoop
	ld a,(menuX)
	or a
	jr nz,UnitMenuLeftNotEnd
	ld a,3
UnitMenuLeftNotEnd:
	dec a
UnitMenuLeftRight:
	cp 2
	jr nz,UnitMenuLeftRightNotPre
	ld a,(menuPrefix)
	or a
	ld a,2
	jr z,UnitMenuLeftRightNotPre
	ld a,(menuX)
	xor 1
UnitMenuLeftRightNotPre:
	ld (menuX),a

	ld hl,0
	ld (menuY),hl
	jp UnitMenuRedrawAll
UnitMenuRight:
	ld a,(menuSub)
	or a
	jp nz,UnitMenuKeyLoop
	ld a,(menuX)
	cp 2
	jr nz,UnitMenuRightNotEnd
	ld a,$ff
UnitMenuRightNotEnd:
	inc a
	jr UnitMenuLeftRight

UnitMenuAlphaUp:
	ld a,(menuZ)
	sub 6
	jr nc,UnitMenuAlphaUp_ZOK
	xor a
UnitMenuAlphaUp_ZOK:
	ld (menuZ),a
	ld a,(menuY)
	sub 6
	jr nc,UnitMenuAlphaUp_YOK
	xor a
UnitMenuAlphaUp_YOK:
	ld (menuY),a
	jp UnitMenuRedrawItemsOnly

UnitMenuAlphaDown:
	ld a,(menuBot)
	ld b,a
	ld a,(menuZ)
	add a,6
	cp b
	jr c,UnitMenuAlphaDown_ZOK
	ld a,b
	dec a
UnitMenuAlphaDown_ZOK:
	ld (menuZ),a
	ld a,(menuY)
	add a,6
	cp b
	jr c,UnitMenuAlphaDown_YOK
	ld a,b
	dec a
UnitMenuAlphaDown_YOK:
	ld (menuY),a
	jp UnitMenuRedrawItemsOnly

UnitMenuKeyAlph:
	sub k0
	ld c,a
	ld hl,(menuPtr)
	ld b,(hl)
	inc hl
UnitMenuKeyAlphLoop:
	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
UnitMenuKeyAlphSkip:
	inc de
	ld a,(de)
	cp $bb
	jr nz,UnitMenuKeyAlph1B
	inc de
	ld a,(de)
	push de
	ld d,Alphabet2_H
	ld e,a
	ld a,(de)
	pop de
	or a
	jr nz,UnitMenuKeyAlph2B
	jr UnitMenuKeyAlphSkip
UnitMenuKeyAlph1B:
	sub '0'
	jr nz,UnitMenuKeyAlphNotZero
	ld a,10
UnitMenuKeyAlphNotZero:
	jr c,UnitMenuKeyAlphSkip
	cp 11
	jr c,UnitMenuKeyAlph2B
	add a,11+'0'-'A'
	cp 11
	jr c,UnitMenuKeyAlphSkip
	cp 37
	jr nc,UnitMenuKeyAlphSkip
UnitMenuKeyAlph2B:
	cp c
	jr nc,UnitMenuKeyAlphDone
	djnz UnitMenuKeyAlphLoop
	inc b
UnitMenuKeyAlphDone:
	ld a,(menuBot)
	sub b
	ld (menuZ),a
	ld (menuY),a
	jp UnitMenuRedrawItemsOnly

UnitMenuHelp:
	ld a,(menuY)
	ld c,a
	ld b,0
	ld hl,(menuPtr)
	inc hl
	add hl,bc
	add hl,bc
	ld e,(hl)
	inc hl
	ld a,(hl)
	or a
	jp z,UnitMenuKeyLoop
	ld d,a
	push de
	push hl
	ld hl,0
	ld (curRow),hl
	B_CALL _ClrLCDFull
	pop hl
	dec hl
	call UnitMenuDrawItemText

	pop hl
	ld c,(hl)
	add hl,bc
	inc hl
	ld a,(menuPrefix)
	or a
	jr nz,UnitMenuHelp_NoPrefix
	ld a,(menuX)
	dec a
	jr z,UnitMenuHelp_Prefix
UnitMenuHelp_NoPrefix:
	inc hl
	ld a,(hl)
	call PutBCD_H
	ld a,'.'
	B_CALL _PutC
	ld a,(hl)
	call PutBCD_L
	inc hl
	ld b,4
UnitMenuHelp_NumLoop:
	ld a,(hl)
	call PutBCD_H
	ld a,(hl)
	call PutBCD_L
	inc hl
	djnz UnitMenuHelp_NumLoop
	ld a,Lexponent
	B_CALL _PutC
	ld bc,-6
	add hl,bc
	ld a,(hl)
	sub $80
	jr nc,UnitMenuHelp_ExpPos
	ld c,a
	ld a,Lneg
	B_CALL _PutC
	ld a,c
	neg
UnitMenuHelp_ExpPos:
	push hl
	ld de,localTokStr
	push de
	call PrintInt
	xor a
	ld (de),a
	pop hl
	call PutS
	pop hl
	ld bc,8
	add hl,bc
	ld de,UnitMenuHelpPosTab
	ld b,7
UnitMenuHelp_MainLoop:
	ld a,(de)
	ld (curRow),a
	inc de
	ld a,(de)
	ld (curCol),a
	inc de
	ld a,(hl)
	call PutSmallUnitExp
	inc hl
	djnz UnitMenuHelp_MainLoop
	ld hl,3
	ld (curRow),hl
	ld hl,UnitMenuHelpUnitStrs
	call PutS
UnitMenuHelp_Std:
	call Rawkey_Sub_Getkey
	ld a,(menuSub)
	or a
	jp z,UnitMenuRedrawAll
	ld l,a
	jp UnitMenuSubReset

UnitMenuHelp_Prefix:
	B_CALL _NewLine
	ld b,5
UnitMenuHelp_PrefixLp:
	ld a,' '
	B_CALL _PutC
	djnz UnitMenuHelp_PrefixLp
	ld a,(hl)
	cp $80
	jr c,UnitMenuHelp_PrefixPos
	ld a,Lneg
	B_CALL _PutC
	ld a,(hl)
	neg
UnitMenuHelp_PrefixPos:
	ld de,localTokStr
	call PrintInt
	xor a
	ld (de),a
	ld hl,localTokStr
	call PutS
	B_CALL _NewLine
	ld hl,UnitMenuHelpTenStr
	call PutS
	jr UnitMenuHelp_Std

PutSmallUnitExp:
	push de
	sub $50
	daa
	jr nc,PutSmallUnitExp_Pos
	add a,$50
	daa
	ld c,a
	ld a,$D2
	B_CALL _PutC
	ld a,$50
	sub c
	daa
PutSmallUnitExp_Pos:
	ld c,a
	rrca
	rrca
	rrca
	rrca
	and $0f
	jr z,PutSmallUnitExp_2
	add a,$80
	B_CALL _PutC
PutSmallUnitExp_2:
	ld a,c
	and $0f
	add a,$80
	B_CALL _PutC
	pop de
	ret

PutBCD_H:
	rrca
	rrca
	rrca
	rrca
PutBCD_L:
	and $0f
	add a,'0'
	B_CALL _PutC
	ret

	;; TITLE
	;; 1.234567890e+XX
	;;  123   123  123
	;; m    kg    s
	;;  123  123   123
	;; A    K    Cd
	;;    123
	;; mol

UnitMenuHelpPosTab:
	.db 2,2
	.db 2,8
	.db 2,13
	.db 4,2
	.db 4,7
	.db 6,4
	.db 4,13

UnitMenuHelpUnitStrs:
	.db " m    kg    s",$D6,$D6
	.db " A    K    Cd",$D6,$D6
	.db " mol",0

UnitMenuHelpTenStr:
	.db "   10",0

UnitMenuSub:
	dec hl
	ld a,(hl)
	ld hl,0
	ld (menuY),hl
	ld (menuSub),a
	ld l,a
UnitMenuSubReset:
	cp SUB_ALPH
; 	jr c,UnitMenuSub_NotAlph
; 	set shiftAlpha,(iy+shiftFlags)
UnitMenuSub_NotAlph:
	ld h,0
	add hl,hl
	add hl,hl
	ld bc,UnitMenuSubs
	add hl,bc
	ld e,(hl)
	inc hl
	ld d,(hl)
	ld (menuPtr),de
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	ld hl,0
	ld (curRow),hl
	ex de,hl
	set textInverse,(iy+textFlags)
	call PutS_Upper
	res textInverse,(iy+textFlags)
UnitMenuSubLoop:
	ld a,' '
	B_CALL _PutC
	ld a,(curCol)
	or a
	jr nz,UnitMenuSubLoop

	ld a,(menuPrefix)
	or a
	jp z,UnitMenuRedrawItemsOnly
	
	call UnitMenuDrawPrefix	
	jp UnitMenuRedrawItemsOnly


Rawkey_Sub_Getkey:
	;; Hooray for kludges!
	res onInterrupt,(iy+onFlags)
	ld hl,(menuX)
	push hl
	ld hl,(menuZ)
	push hl
	ld hl,(menuPtr)
	push hl
	ld hl,(menuBot)
	push hl
	ld hl,(menuPrefix)
	push hl
	B_CALL _GetKey
	pop hl
	ld (menuPrefix),hl
	pop hl
	ld (menuBot),hl
	pop hl
	ld (menuPtr),hl
	pop hl
	ld (menuZ),hl
	pop hl
	ld (menuX),hl
	ret


UnitMenuDrawPrefix:
	;; A = prefix to draw
	ld hl,12*256
	ld (curRow),hl
	add a,a
	ld hl,MenuPrefix-1
	ld c,a
	ld b,0
	add hl,bc
UnitMenuDrawItemText:
	ld e,(hl)
	inc hl
	ld a,(hl)		; Get address of token string to be displayed.
	inc hl
	push hl
	 ld d,a
	 or a			; if high byte=0, is a reference to a submenu
	 jr nz,UnitMenuDrawItemToken
	 ex de,hl
	 add hl,hl
	 inc hl
	 add hl,hl
	 ld de,UnitMenuSubs
	 add hl,de
	 ld e,(hl)
	 inc hl
	 ld d,(hl)
	 ex de,hl
	 call PutS
	 ld a,Lellipsis
	 B_CALL _PutC
	 jr UnitMenuDrawItemX
UnitMenuDrawItemToken:
	 ld a,' '
	 B_CALL _PutC
	 ex de,hl
	 ld b,(hl)		; Length of string
	 inc hl
UnitMenuDrawItemTokenLoop:
	 ld a,(hl)
	 inc hl
	 ld d,0
	 ld e,a
	 B_CALL _IsA2ByteTok
	 jr nz,UnitMenuDrawItemTokenOne
	 ld d,a
	 ld e,(hl)
	 inc hl
	 dec b
UnitMenuDrawItemTokenOne:
	 push bc
	  push hl
	   B_CALL _PutTokString
	   pop hl
	  pop bc
	 djnz UnitMenuDrawItemTokenLoop
	 ld de,-EndOfPrefixStrings
	 add hl,de
	 jr c,UnitMenuDrawItemX
	 ld a,'-'
	 B_CALL _PutC
UnitMenuDrawItemX:
	 call EraseNewLine
	 pop hl
	ret

GetMenuChar:
	ld a,(curRow)
	dec a
	jr z,GetMenuCharTop
	cp 6
	ld a,':'
	ret nz
	ld a,c
	inc a
	cp b
	ld a,':'
	ret z
	ld a,$1f
	ret
GetMenuCharTop:
	ld a,(menuZ)
	or a
	ld a,':'
	ret z
	ld a,$1e
	ret

UnitMenuNoAVStr:
	.db "UnitMode appvar",$D6
	.db " not found or",$D6
	.db " invalid. UNIT",$D6
	.db " menu will not",$D6
	.db " function.",$D6
	.db "Please reinstall"
	.db " the app.",$D6
	.db "- Press a key -",0


;;; Special keys to be returned by the menu (again taken from Omnicalc)
SpecialKeys:
	.db kClear,kAppsMenu,kPrgm,kZoom,kDraw,kSPlot,kStat,kMath
	.db kTest,kChar,kVars,kMem,kMatrix,kDist,kAngle,kList,kCalc
	.db kCatalog,kGraph,kMode,kWindow,kYequ,kTable,kTblSet
	.db kStatP,kFormat,kTrace,kQuit
