;;; -*- mode: TI-Asm; ti-asm-listing-file: "units.lst" -*-
;;;
;;;	Unit Operations Flash App for TI-83+
;;; 
;;;	Copyright 2003 Benjamin Moody
;;;
;;;  This program is free software; you can redistribute
;;;  and/or modify it under the terms of the GNU General
;;;  Public License as published by the Free Software
;;;  Foundation; either version 2 of the License, or (at
;;;  your option) any later version.
;;;
;;;  This program is distributed in the hope that it will be
;;;  useful, but WITHOUT ANY WARRANTY; without even the
;;;  implied warranty of MERCHANTABILITY or FITNESS FOR A
;;;  PARTICULAR PURPOSE. See the GNU General Public License
;;;  for more details.
;;;
;;;  You should have received a copy of the GNU General
;;;  Public License along with this program; if not, write
;;;  to the Free Software Foundation, Inc., 59 Temple Place
;;;  - Suite 330, Boston, MA 02111 USA.
;;;

;;; read.asm: SetOP1Units and related stuff

SetOP1Units_NO_STRING:
	ld a,(OP1)
	and $1f
	jr z,SetUnitCplx0
	cp CplxObj
	ret nz
SetUnitCplx:
	;; Complex format: +-a.abbccddeeffgg*i*10^-97 OR 0*i
	ld a,(OP2+1)
	cp $80-97
	ret z
	ld a,(OP2+2)
	or a
	ret nz
SetUnitCplx0:
	ld hl,NoUnits
	ld de,OP2
	ld bc,9
	ldir
	ld a,(OP1)
	or $0c
	ld (OP1),a
	cp a
	ret

SetOP1Units:
;;; Note: this code will use all of OP1-OP6, OP7-OP9.

	ld a,(OP1)
	and $1f
	;; Valid types:	real (dimensionless), complex with flag, string
	jr z,SetUnitCplx0
	cp CplxObj
	jr z,SetUnitCplx
	cp StrngObj
	ret nz
	rst rFindSym
	ret c
	ex de,hl
	ld b,(hl)
	inc hl
	ld a,(hl)
	inc hl
	or a
	;; No string longer than 256 bytes. I'm lazy.
	ret nz

	push hl
	push bc
	call OP1ToOP9
	ld hl,One
	ld de,OP1
	call Mov22
	pop bc
	pop hl
	xor a
	ld (parseNumDen),a

;;; Input string format
;;; Valid strings:
;;;   Literal number: 0-9 . E -
;;;   Unit string as given in table
;;;   Space: subsequent strings considered in numerator
;;;   /: subsequent strings in denominator
;;;   Dot or *: separate without changing num/den.
;;;   To do: Parens, exponents, std order of operations for *,/
;;;   Actual math so that we can use this as homescreen entry parser!

	ld a,b
	or a
	jp z,SetOP1UnitsDone

SetOP1UnitsLoop:
	push hl
	 ld c,0
	 ld a,(hl)
	 cp tChs
	 jr z,SetOP1UnitsFoundSep
SetOP1UnitsSearch:
	 ld a,(hl)
	 cp tSpace
	 jr z,SetOP1UnitsFoundSep
	 cp tDiv
	 jr z,SetOP1UnitsFoundSep
	 cp tDotIcon
	 jr z,SetOP1UnitsFoundSep
	 cp tMul
	 jr z,SetOP1UnitsFoundSep
	 cp tSqr
	 jr z,SetOP1UnitsFoundSep
	 cp tCube
	 jr z,SetOP1UnitsFoundSep
	 cp tPower
	 jr z,SetOP1UnitsFoundSep
	 inc hl
	 inc c
	 djnz SetOP1UnitsSearch
	 xor a
SetOP1UnitsFoundSep:
	 ex (sp),hl		; save end of substring, return to beginning
	 push bc
	  push af
	   ld b,c
	   call FindUnitString
	   jp nz,SetOP1UnitsBadString
	   ;; We did find a unit: HL->data, DE->name
	   push hl
	    push de
	     ld a,(hl)
	     ld c,a
	     ld b,0
	     add hl,bc
	     ld bc,1+8
	     add hl,bc
	     ld de,NoUnits+2
	     ld b,7
	     call CompStrsN	; It is REALLY BAD to have dimensionless
	     pop hl		; preferences.
	    dec hl
	    call AddPreference	; which checks the zero flag
	    pop hl

SetOP1UnitsRepeat:
	;; HL -> data of a unit to multiply or divide by.
	   push hl
	    ld de,OP3+1
	    call Mov8
	    ld de,OP4+2
	    call Mov7
	    xor a
	    ld (OP3),a
	    ld (OP4),a
	    ld a,$80-97
	    ld (OP4+1),a

SetOP1UnitsMultValue:
	;; We will now multiply or divide by OP3-4.
	    ld a,(parseNumDen)
	    or a
	    jr z,SetOP1UnitsNumerator
	    call SubUnitsOP2OP4
	    call OP3ToOP2
	    B_CALL _FPDiv
	    ld a,(unitPrefix)
	    ld b,a
	    ld a,(OP1+1)
	    sub b
	    ld (OP1+1),a
	    jr SetOP1UnitsDenominator
SetOP1UnitsNumerator:
	    call AddUnitsOP2OP4
	    call OP3ToOP2
	    B_CALL _FPMult
	    ld a,(unitPrefix)
	    ld b,a
	    ld a,(OP1+1)
	    add a,b
	    ld (OP1+1),a
SetOP1UnitsDenominator:
	    call OP4ToOP2
	    pop hl
	   pop af
	  cp tSqr
	  jr z,SetOP1UnitsSquare
	  cp tCube
	  jr z,SetOP1UnitsCube
	  cp tPower
	  jr z,SetOP1UnitsPower
	  cp tChs
	  jr z,SetOP1UnitsChs
	  cp tSpace
	  jr z,SetOP1UnitsSetNum
	  cp tDiv
	  jr nz,SetOP1UnitsDontSet
	  ld a,1
	  jr SetOP1UnitsSetDen
SetOP1UnitsSetNum:
	  xor a
SetOP1UnitsSetDen:
	  ld (parseNumDen),a
SetOP1UnitsDontSet:
	  pop bc
	 pop hl
	ld a,b
	or a
	jr z,SetOP1UnitsDone
	inc hl
	dec b
	jp nz,SetOP1UnitsLoop
	jr SetOP1UnitsDone
SetOP1UnitsPop3Done:
	pop hl
	pop hl
	pop hl
SetOP1UnitsDone:
	call CplxOP1
	cp a
	ret
SetOP1UnitsBadString:
	call OP9ToOP1
	jp Pop3RetNZ

SetOP1UnitsChs:
	  ld a,(OP1)
	  xor $80
	  ld (OP1),a
	  jr SetOP1UnitsDontSet

SetOP1UnitsSquare:
	  pop bc
	 pop de
	inc de
	ld a,(de)
	dec b
	push de
	 push bc
	  push af
	   jp SetOP1UnitsRepeat

SetOP1UnitsCube:
	  ld a,tSqr
	  push af
	   jp SetOP1UnitsRepeat

SetOP1UnitsPower:
	  pop bc		; count of characters left
	 ex (sp),hl		; get position in string, save unit data
	 ld c,0
SetOP1UnitsPowerLoop:
	;; read exponent
	 inc hl
	 dec b
	 jr z,SetOP1UnitsPowerDone
	 ld a,(hl)
	 sub t0
	 jr c,SetOP1UnitsPowerDone
	 cp 10
	 jr nc,SetOP1UnitsPowerDone
	 ld d,a
	 ld a,c
	 add a,a
	 add a,a
	 add a,c
	 add a,a
	 add a,d
	 ld c,a
	 jr SetOP1UnitsPowerLoop
SetOP1UnitsPowerDone:
	 ld a,(hl)
	 ex (sp),hl		; save position in string
	 push bc		; count of chars left
	  push af		; and the next character to handle
	   ld a,c
	   or a
	   jp z,ErrSyntax
	   dec a
	   push af		; save exponent
	    push hl		; and unit data
	     ld de,OP7
	     call Mov22FrOP1	; save current value
	     pop hl
	    ld de,OP1+1
	    call Mov8
	    pop af
	   push af
	    push hl
	     B_CALL _SetXXOP2
	     B_CALL _YToX
	     ld de,OP3
	     call Mov11FrOP1
	     pop hl
	    ld de,OP4+2
	    call Mov7
	    pop af
	   ld e,a
	   call MulUnit
	   ld hl,OP7
	   call Mov22OP1

	   ld hl,u_one
	   push hl
	    jp SetOP1UnitsMultValue

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



FindUnitString:
	;; HL -> string to find
	;; B = length
	;; Output (if successful) DE->start of unit entry, HL->end of name
	ld a,b
	or a
	jr z,FindUnitStringOne
	ld a,(hl)
	cp tDecPt
	jr z,FindUnitString_Number
	cp tee
	jr z,FindUnitString_Number
	cp t0
	jr c,FindUnitString_NotNum
	cp t9+1
	jr c,FindUnitString_Number
FindUnitString_NotNum:

	xor a
	ld (unitPrefix),a
	ex de,hl
	ld hl,NullPrefix
	ld (lastPrefixPtr),hl
FindUnitStringAgain:
	ld hl,UnitStrings
FindUnitStringLoop:
	push de
	push bc
	ld a,(hl)
	or a
	jr z,FindUnitString_NotInt
	inc hl
	cp b
	jr nz,FindUnitStringSkip
	push hl
	call CompStrsN
	jr z,FindUnitStringFound
	pop de
	ld a,b
FindUnitStringSkip:
	ld b,a
FindUnitStringLoop0:
	inc hl
	djnz FindUnitStringLoop0
	ld de,15		; Skip past the scale and unit
	add hl,de
	pop bc
	pop de
	jr FindUnitStringLoop
FindUnitString_NotInt:

; 	ld hl,_LdA_CHL
; 	ld de,LdA_CHL
; 	ld bc,LdA_CHL_Size
; 	ldir
; 	call LoadCustom
; 	pop bc
; 	pop de
; 	ld c,a
; FindUnitStringCustomLoop:
; 	push de
; 	call LdA_CHL
; 	push bc
; 	or a
; 	jr z,FindUnitStringFailed
; 	cp b
; 	jr nz,FindUnitStringCustomSkip
; 	push bc
; 	push de
; 	ld de,customUnit
; 	ld a,c
; 	ld c,b
; 	ld b,0
; 	B_CALL _FlashToRAM
; 	pop de
; 	pop bc
; 	push hl
; 	ld hl,customUnit
; 	call CompStrsN
; 	jr z,FindUnitStringCustomFound
; 	pop hl
; 	pop bc
; 	ld de,15
; 	call AddCHL_DE
; 	pop de
; 	jr FindUnitStringCustomLoop
; FindUnitStringCustomSkip:
; 	ld a,b
; 	add a,15
; 	ld e,a
; 	ld d,0
; 	pop bc
; 	call AddCHL_DE
; 	pop de
; 	jr FindUnitStringCustomLoop


FindUnitStringFailed:
	pop bc
	pop de
	call CheckPrefixString
	jr z,FindUnitStringAgain
	ret
FindUnitStringCustomFound:

FindUnitStringFound:
	pop de
	pop bc
	pop bc
	ret

FindUnitStringOne:
	;; If we have an empty string (e.g. multiple spaces)
	;; This does lead to illegal strings being accepted, but does this matter?
	ld hl,u_one
	ld de,NonPreferrableName
	cp a
	ret

FindUnitString_Number:
	;; HL -> string to parse
	;; B = length
	push bc
	push hl
	ld hl,customUnit+1
	push hl
	ld de,customUnit+2
	ld (hl),$0f		; mark un-asssigned digits
	call Mov7
	pop hl
	pop de
	pop bc
	ld c,$7f		; current exponent
FindNumberLoop:
	ld a,(de)
	cp tDecPt
	jr z,FindNumberDecPt
	cp tee
	jr z,FindNumberEE
	sub t0
	jp c,RetNZ
	cp 10
	jp nc,RetNZ
	rld
	cp $0f
	jr nz,FindNumberNoInc
	inc hl
FindNumberNoInc:
	inc de
	inc c
	djnz FindNumberLoop
FindNumberDone:
	ld a,c
FindNumberDoneA:
	ld (customUnit),a
	ld hl,customUnit+1
	ld b,7
FindNumberCleanLoop:
	ld a,(hl)
	cp $f0
	jr nc,FindNumberCleanH
	cp $0f
	jr nz,FindNumberAlreadyClean
	ld (hl),0
	jr FindNumberAlreadyClean
FindNumberCleanH:
	xor a
	rld
FindNumberAlreadyClean:
	inc hl
	djnz FindNumberCleanLoop
	ld hl,NoUnits+2
	ld de,customUnit+8
	call Mov7
	ld hl,customUnit
	ld de,NonPreferrableName
	cp a
	ret

FindNumberDecPt:
	inc de
	dec b
	jr z,FindNumberDone
FindNumberDecLoop
	ld a,(de)
	cp tDecPt
	jp z,RetNZ
	cp tee
	jr z,FindNumberEE
	sub t0
	jp c,RetNZ
	cp 10
	jp nc,RetNZ
	rld
	cp $0f
	jr nz,FindNumberDecNoInc
	inc hl
FindNumberDecNoInc:
	inc de
	djnz FindNumberDecLoop
	jr FindNumberDone

FindNumberEE:
	inc de
	dec b
	jr z,FindNumberDone
	ld h,0
	ld a,(de)
	cp tChs
	jr z,FindNumberEENeg
	call ParseEE
	ret nz
	ld a,h
	add a,c
	jr FindNumberDoneA
FindNumberEENeg:
	inc de
	dec b
	jr z,FindNumberDone
	call ParseEE
	ret nz
	ld a,c
	sub h
	jr FindNumberDoneA

ParseEE:
	ld a,(de)
	sub t0
	ret c
	cp 10
	jp nc,RetNZ
	ld l,a
	ld a,h
	add a,a
	add a,a
	add a,h
	add a,a
	add a,l
	ld h,a
	inc de
	djnz ParseEE
	cp a
	ret


CheckPrefixString:
	ld hl,PrefixStrings
CheckPrefixStringLoop:
	push de
	push bc
	ld (lastPrefixPtr),hl
	ld a,(hl)
	inc hl
	or a
	jr z,CheckPrefixStringDone
	cp b
	ld b,a
	ld c,a
	jr nc,CheckPrefixStringLoop0
	call CompStrsN
	jr z,CheckPrefixStringFound
CheckPrefixStringLoop0:
	inc hl
	djnz CheckPrefixStringLoop0
	inc hl
	pop bc
	pop de
	jr CheckPrefixStringLoop
CheckPrefixStringDone:
	pop bc
	pop de
	or 1
	ret
CheckPrefixStringFound:
	ld a,(hl)
	ld (unitPrefix),a
	pop af
	pop hl
	sub c
	ld b,a
	cp a
	ret


AddPreference:
	;; HL -> unit
	;; (lastPrefixPtr) -> prefix
	;; If Z set, only allow preset conversion

	push af

	ld (tempUnitPtr),hl

	ld de,convab
	call CpHLDE
	jp z,AddPreferenceSetAllBase
	ld de,convde
	call CpHLDE
	jp z,AddPreferenceSetDefault
	ld de,convam
	call CpHLDE
	jp z,AddPreferenceSetAmerican

	ld de,NO_PREFER
	call CpHLDE
	jp c,PopRet

	pop af
	ret z

	push hl

	 ld hl,4
	 B_CALL _EnoughMem
	 jp c,PopRet

	 call LoadAppVar
	 ld (appVarPtr),de

	 ld hl,4
	 B_CALL _InsertMem
	 ex de,hl
	 pop de
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ld de,(lastPrefixPtr)
	ld (hl),e
	inc hl
	ld (hl),d

	ld de,-3-appVarFixedLen-2 ; move back to size word
	add hl,de
	ld a,(hl)
	add a,4
	ld (hl),a
	jr nc,AddPreference_SizeNC
	inc hl
	inc (hl)
	dec hl
AddPreference_SizeNC:
	ld de,2+appVarFixedLen+4 ; move forward to the second entry in the list
	add hl,de
	push hl
	 ld hl,(tempUnitPtr)
	 ld c,(hl)
	 ld b,0
	 add hl,bc
	 ld bc,1+8
	 add hl,bc
	 ld (tempUnitPtr),hl
	 pop hl
	;; Now we search for any equivalent units
	;; (We won't do a superunit search, to make this stuff run faster.)
	;; (This might be something we should do at some other time, like
	;;  after execution finishes in the home. However, it's not actually
	;;  all that important, as there are a finite list of possible
	;;  dimensions, and therefore a maximum number of unused preferences
	;;  under the current scheme.)
AddPreferenceLoop:
	;; HL -> preference list
	ld e,(hl)
	inc hl
	ld d,(hl)
	ld a,d
	or e
	ret z
	push hl
	 ld hl,(tempUnitPtr)
	 ex de,hl
	 ld c,(hl)
	 ld b,0
	 add hl,bc
	 ld bc,1+8
	 add hl,bc
	 ld b,7
	 call CompStrsN
	 jr z,AddPreferenceFound
	 pop hl
	inc hl
	inc hl
	inc hl
	jr AddPreferenceLoop
AddPreferenceFound:
	 pop hl
	dec hl
	ld de,4
	B_CALL _DelMem
	ld hl,(appVarPtr)
	ld de,-appVarFixedLen-2
	add hl,de
	ld a,(hl)
	sub 4
	ld (hl),a
	ret nc
	inc hl
	dec (hl)
	ret

	
AddPreferenceSetAllBase:
	ld hl,AllBase
	ld de,AllBaseLength
	jr AddPreferencePreset
AddPreferenceSetAmerican:
	ld hl,American
	ld de,AmericanLength
	jr AddPreferencePreset
AddPreferenceSetDefault:
	ld hl,appVarDefPre
	ld de,appVarDefPreLen
AddPreferencePreset:
	push hl
	push de

	ld hl,ClearHooks
	ld de,backupHooks
	call Mov9
	
	ld hl,appVarName
	rst rMov9ToOP1
	B_CALL _ChkFindSym
	jr c,AddPreferencePresetNoDel
	push hl
	push de
	push bc
	ld hl,2+appVarSigLen
	add hl,de
	ld de,backupHooks
	call Mov9
	pop bc
	pop de
	pop hl
	B_CALL _DelVar
AddPreferencePresetNoDel:
	pop hl
	push hl
	ld bc,appVarSigLen+9
	add hl,bc
	B_CALL _CreateAppVar
	inc de
	inc de
	ld hl,appVarSig
	call Mov6
	ld hl,backupHooks
	call Mov9
	pop bc
	pop hl
	ldir

	pop af
	ret

AllBase:
	.db 0
	.dw meter,NullPrefix
	.dw gram,kilo
	.dw second,NullPrefix
	.dw ampere,NullPrefix
	.dw kelvin,NullPrefix
	.dw mole,NullPrefix
	.dw candel,NullPrefix
	.dw 0,0
AllBaseLength = $-AllBase

American:
	.db 0
	.dw gallon,NullPrefix
	.dw foot,NullPrefix
	.dw pound,NullPrefix
	.dw second,NullPrefix
	.dw ampere,NullPrefix
	.dw degR,NullPrefix
	.dw mole,NullPrefix
	.dw candel,NullPrefix
	.dw 0,0
AmericanLength = $-American
