;;; -*- mode: TI-Asm; ti-asm-listing-file: "units.lst" -*-
;;;
;;;	Unit Operations Flash App for TI-83+
;;; 
;;;	Copyright 2003 Benjamin Moody
;;;
;;;  This program is free software; you can redistribute
;;;  and/or modify it under the terms of the GNU General
;;;  Public License as published by the Free Software
;;;  Foundation; either version 2 of the License, or (at
;;;  your option) any later version.
;;;
;;;  This program is distributed in the hope that it will be
;;;  useful, but WITHOUT ANY WARRANTY; without even the
;;;  implied warranty of MERCHANTABILITY or FITNESS FOR A
;;;  PARTICULAR PURPOSE. See the GNU General Public License
;;;  for more details.
;;;
;;;  You should have received a copy of the GNU General
;;;  Public License along with this program; if not, write
;;;  to the Free Software Foundation, Inc., 59 Temple Place
;;;  - Suite 330, Boston, MA 02111 USA.
;;;

;;; unitdata.asm: Unit strings and values

PrefixStrings:
	;; If the initial pass through the unit strings fails, we should check
	;; for a valid prefix and test again with that prefix. If the second
	;; check fails, send the string back to the OS.
yotta:	.db 1,'Y',24		; yotta - 10^24
zetta:	.db 1,'Z',21		; zetta - 10^21
exa:	.db 1,'E',18		; exa - 10^18
peta:	.db 1,'P',15		; peta - 10^15
tera:	.db 1,'T',12		; tera - 10^12

giga:	.db 1,'G',9		; giga - 10^9
mega:	.db 1,'M',6		; mega - 10^6
kilo:	.db 2,$bb,tLk,3		; kilo - 10^3
hecto:	.db 2,$bb,tLh,2		; hecto - 10^2
deka:	.db 4,$bb,tLd,$bb,tLa,1	; deka - 10^1

deci:	.db 2,$bb,tLd,-1	; deci - 10^-1
centi:	.db 2,$bb,tLc,-2	; centi - 10^-2
milli:	.db 2,$bb,tLm,-3	; milli - 10^-3
micro:	.db 2,$bb,tLmu,-6	; micro - 10^-6
nano:	.db 2,$bb,tLsmalln,-9	; nano - 10^-9

pico:	.db 2,$bb,tLp,-12	; pico - 10^-12
femto:	.db 2,$bb,tLf,-15	; femto - 10^-15
atto:	.db 2,$bb,tLa,-18	; atto - 10^-18
zepto:	.db 2,$bb,tLz,-21	; zepto - 10^-21
yocto:	.db 2,$bb,tLy,-24	; yocto - 10^-24

NullPrefix:
	.db 0,0			; This one won't be considered when inputting,
				; but when outputting, this is the default
				; preferred prefix.

EndOfPrefixStrings:

UnitStrings:
	;; Form is token-string, factor (8 bytes), unit string (7 bytes)
	;; (both are assumed positive)

	;; Weird temperature scales should be processed separately...
	;; I haven't done this.

	;; Note that the list has two parts. The second part is used for
	;; formatting, when the user has no applicable preferences for the unit
	;; specified. The first, larger part is used only when reading strings.

	;; And be forewarned: I was able to find values and brief descriptions
	;; of each of the following constants. That should not imply that I
	;; understand what they all mean :)
	;; Don't hesitate to correct me - an inaccurate reference is far worse
	;; than not having one at all, and I certainly hope this program will
	;; be useful.


;;; These aren't actually designed to be used as units, but flags for
;;; conversion (they load a deault preference list)

convde:	.db 8,tGT,"DEFAULT"	; Uses all default units
u_one:	.db $80,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$50

convab:	.db 5,tGT,"BASE"	; Uses only base units
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$50

convam:	.db 3,tGT,"US"		; Uses American units (somewhat)
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$50


;;; People generally don't like to see their temperatures expressed in
;;; terms of zero degrees...

__ZdegF: .db 4,tColon,'0',tFromDeg,'F'	; origin of Fahrenheit temperature
	.db $82,$25,$53,$72,$22,$22,$22,$22,$50,$50,$50,$50,$51,$50,$50

__ZdegC: .db 4,tColon,'0',tFromDeg,'C'	; origin of Celsius temperature
	.db $82,$27,$31,$50,0,0,0,0,	$50,$50,$50,$50,$51,$50,$50

NonPreferrableName:		; Used for e.g. literal numbers

ver:	.db 7,"VERSION"
	.db $7f,$10*VMinor,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$50


	;; DIMENSIONLESS

; percnt:	.db 2,$bb,tPercent	; %	percent
; 	.db $7e,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$50
ppm:	.db 6,$bb,tLp,$bb,tLp,$bb,tLm ; ppm
	.db $7a,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$50
ppb:	.db 6,$bb,tLp,$bb,tLp,$bb,tLb ; ppb
	.db $77,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$50
ppt:	.db 6,$bb,tLp,$bb,tLp,$bb,tLsmallt
	.db $74,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$50
proof:	.db 10,$bb,tLp,$bb,tLsmallr,$bb,tLo,$bb,tLo,$bb,tLf
	.db $7d,$50,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$50
karat:	.db 3,'K',$bb,tLsmallt
	.db $7e,$41,$66,$66,$66,$66,$66,$66,$50,$50,$50,$50,$50,$50,$50

;;; Constants

__G:	.db 2,tColon,'G'	; G	Gravitational constant
	.db $75,$66,$73,0,0,0,0,0,	$53,$49,$48,$50,$50,$50,$50

__c:	.db 3,tColon,$bb,tLc	; :c	speed of light
	.db $88,$29,$97,$92,$45,$80,0,0,$51,$50,$49,$50,$50,$50,$50

__mu0:	.db 4,tColon,$bb,tLmu,'0'	; permeability
	.db $7a,$12,$56,$63,$71,0,0,0,	$51,$51,$48,$48,$50,$50,$50

__eps0:	.db 4,tColon,$bb,tLepsilon,'0'	; permittivity
	.db $74,$88,$54,$18,$78,0,0,0,	$47,$49,$54,$52,$50,$50,$50

__e:	.db 3,tColon,$bb,tLsmalle	; e	electron charge
	.db $6d,$16,$02,$17,$64,$62,0,0,$50,$50,$51,$51,$50,$50,$50

__h:	.db 3,tColon,$bb,tLh	; h	Planck constant
	.db $5e,$66,$26,$06,$87,$60,0,0,$52,$51,$49,$50,$50,$50,$50

__hbar:	.db 6,tColon,$bb,tLh,"BAR", ;	A useful multiple of the above
	.db $5e,$10,$54,$57,$16,0,0,0,	$52,$51,$49,$50,$50,$50,$50

__kC:	.db 4,tColon,$bb,tLk,'C'	; kC Coulomb constant
	.db $89,$89,$87,$55,$17,$87,0,0,$53,$51,$46,$48,$50,$50,$50


__g:	.db 3,tColon,$bb,tLsmallg ; :g	gravitational accel (~)
	.db $80,$98,$06,$65,0,0,0,0,	$51,$50,$48,$50,$50,$50,$50

__N_A:	.db 3,tColon,"NA"	; NA	Avogadro's number
	.db $97,$60,$22,$14,$20,0,0,0,	$50,$50,$50,$50,$50,$49,$50

__R:	.db 2,tColon,'R'	; R	Ideal gas constant
	.db $80,$83,$14,$47,$20,0,0,0,	$52,$51,$48,$50,$49,$49,$50

__k:	.db 3,tColon,$bb,tLk	; k	Boltzmann constant
	.db $69,$13,$80,$65,$03,0,0,0,	$52,$51,$48,$50,$49,$50,$50


__Rinf:	.db 8,tColon,'R',$bb,tLi,$bb,tLsmalln,$bb,tLf
	.db $86,$10,$97,$37,$31,$56,$80,0,	$49,$50,$50,$50,$50,$50,$50

__R_H:	.db 3,tColon,"RH"
	.db $86,$10,$96,$77,$60,0,0,0,	$49,$50,$50,$50,$50,$50,$50

__alph:	.db 3,tColon,$bb,tLalpha
	.db $7d,$72,$97,$35,$25,$33,0,0,$50,$50,$50,$50,$50,$50,$50

__m_P:	.db 4,tColon,$bb,tLm,'P' ; mP	Planck mass
	.db $78,$21,$76,$70,0,0,0,0,	$50,$51,$50,$50,$50,$50,$50

__t_P:	.db 4,tColon,$bb,tLsmallt,'P' ; tP	Planck time
	.db $54,$53,$90,$58,$73,0,0,0,	$50,$50,$51,$50,$50,$50,$50

__l_P:	.db 4,tColon,$bb,tLl,'P' ; lP	Planck length
	.db $5D,$16,$16,$05,$74,0,0,0,	$51,$50,$50,$50,$50,$50,$50

__me:	.db 5,tColon,$bb,tLm,$bb,tLsmalle ;	Electron mass
	.db $61,$91,$09,$38,$19,0,0,0,	$50,$51,$50,$50,$50,$50,$50

__mp:	.db 5,tColon,$bb,tLm,$bb,tLp ;	Proton mass
	.db $65,$16,$72,$62,$16,0,0,0,	$50,$51,$50,$50,$50,$50,$50

__mn:	.db 5,tColon,$bb,tLm,$bb,tLsmalln ;	Neutron mass
	.db $65,$16,$74,$92,$72,0,0,0,	$50,$51,$50,$50,$50,$50,$50

__mmu:	.db 5,tColon,$bb,tLm,$bb,tLmu ;	Muon mass
	.db $64,$18,$83,$53,$11,0,0,0,	$50,$51,$50,$50,$50,$50,$50

__mal:	.db 5,tColon,$bb,tLm,$bb,tLalpha ;	Alpha mass
	.db $65,$66,$44,$65,$60,0,0,0,	$50,$51,$50,$50,$50,$50,$50

__mde:	.db 5,tColon,$bb,tLm,$bb,tLd ;	Deuteron mass
	.db $65,$33,$43,$58,$31,0,0,0,	$50,$51,$50,$50,$50,$50,$50

__l_C:	.db 4,tColon,$bb,tLlambda,'C' ; Compton wavelength of electron
	.db $74,$24,$26,$31,$02,0,0,0,	$51,$50,$50,$50,$50,$50,$50

__l_Cp:	.db 6,tColon,$bb,tLlambda,'C',$bb,tLp ; Compton wavelength of proton
	.db $71,$13,$21,$40,$98,0,0,0,	$51,$50,$50,$50,$50,$50,$50

__l_Cn:	.db 6,tColon,$bb,tLlambda,'C',$bb,tLsmalln ; of neutron
	.db $71,$13,$19,$59,$09,0,0,0,	$51,$50,$50,$50,$50,$50,$50

__mu_B:	.db 4,tColon,$bb,tLmu,'B'
	.db $68,$92,$40,$09,0,0,0,0,	$52,$50,$50,$51,$50,$50,$50

__mu_N:	.db 4,tColon,$bb,tLmu,'N'
	.db $65,$50,$50,$78,$32,0,0,0,	$52,$50,$50,$51,$50,$50,$50

__mumu:	.db 5,tColon,$bb,tLmu,$bb,tLmu
	.db $66,$44,$90,$44,$81,$30,0,0,$52,$50,$50,$51,$50,$50,$50

__mu_p:	.db 5,tColon,$bb,tLmu,$bb,tLp
	.db $66,$14,$10,$60,$66,$33,0,0,$52,$50,$50,$51,$50,$50,$50

__mu_e:	.db 5,tColon,$bb,tLmu,$bb,tLsmalle
	.db $68,$92,$84,$76,$36,$20,0,0,$52,$50,$50,$51,$50,$50,$50

__mu_n:	.db 5,tColon,$bb,tLmu,$bb,tLn
	.db $65,$96,$62,$36,$40,0,0,0,	$52,$50,$50,$51,$50,$50,$50

__mu_d:	.db 5,tColon,$bb,tLmu,$bb,tLd
	.db $65,$43,$30,$73,$45,$70,0,0,$52,$50,$50,$51,$50,$50,$50

NO_PREFER:


__pi:	.db 2,tColon,tPi
	.db $80,$31,$41,$59,$26,$53,$58,$98,	$50,$50,$50,$50,$50,$50,$50


	;; LENGTH

angstr:	.db 2,$bb,tLcapACaret	; A^	Angstrom	= 1e-10 m
	.db $76,$10,0,0,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50

mil:	.db 6,$bb,tLm,$bb,tLi,$bb,tLl ; Mil		= 1/1000 i
	.db $7b,$25,$40,0,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50

line:	.db 8,$bb,tLl,$bb,tLi,$bb,tLsmalln,$bb,tLsmalle	; Line = 1/12 in
	.db $7d,$21,$16,$66,$66,$66,$66,$66,$51,$50,$50,$50,$50,$50,$50

inch:	.db 4,$bb,tLi,$bb,tLsmalln ;	Inch (intl)	= 2.54 cm
	.db $7e,$25,$40,0,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50

foot:	.db 4,$bb,tLf,$bb,tLsmallt	; ft	Foot	= 12 in
	.db $7f,$30,$48,0,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50

yard:	.db 4,$bb,tLy,$bb,tLd		; yd	Yard	= 3 ft
	.db $7f,$91,$44,0,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50

fathom:	.db 8,$bb,tLf,$bb,tLa,$bb,tLsmallt,$bb,tLh	; Fathom = 6 ft
	.db $80,$18,$28,$80,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50

rod:	.db 4,$bb,tLsmallr,$bb,tLd	; rd	Rod	= 5.5 yd
	.db $80,$50,$29,$20,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50

chain:	.db 4,$bb,tLc,$bb,tLh		; ch	Chain	= 66 ft
	.db $81,$20,$11,$68,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50

flng:	.db 8,$bb,tLf,$bb,tLl,$bb,tLsmalln,$bb,tLsmallg	; Furlong = 40 rd
	.db $82,$20,$11,$68,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50

mile:	.db 4,$bb,tLm,$bb,tLi	; mi	Mile		= 8 flng
	.db $83,$16,$09,$34,$40,0,0,0,	$51,$50,$50,$50,$50,$50,$50

nmile:	.db 5,'N',$bb,tLm,$bb,tLi	; Nautical mile
	.db $83,$18,$52,0,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50
	;; Roughly 1 minute of latitude at the equator

au:	.db 2,"AU"		; AU	Astronomical unit
	.db $8b,$14,$95,$97,$87,$06,$91,0,	$51,$50,$50,$50,$50,$50,$50
	;; Ideal orbital radius of the earth (ignoring the earth's and other
	;; planets' gravity, and assuming a circular orbit with the same
	;; period.)

ltsec:	.db 4,$bb,tLl,$bb,tLs	; ls	Lightsecond
	.db $88,$29,$97,$92,$45,$80,0,0,$51,$50,$50,$50,$50,$50,$50
	;; Distance travelled by light in a second (see definition of
	;; meters!)

ltyr:	.db 4,$bb,tLl,$bb,tLy	; ly	Lightyear
	.db $8f,$94,$60,$73,$05,0,0,0,	$51,$50,$50,$50,$50,$50,$50

parsec:	.db 4,$bb,tLp,$bb,tLc	; pc	Parsec
	.db $90,$30,$85,$67,$76,0,0,0,	$51,$50,$50,$50,$50,$50,$50
	;; Distance to an object with parallax of one arcsecond.

	;; MASS

tonne:	.db 2,$bb,tLsmallt	; t	Tonne		= 1000 kg
	.db $83,$10,0,0,0,0,0,0,	$50,$51,$50,$50,$50,$50,$50

amu:	.db 2,$bb,tLu		; u	Atomic mass unit
	.db $65,$16,$60,$53,$87,$30,0,0,$50,$51,$50,$50,$50,$50,$50

pound:	.db 4,$bb,tLl,$bb,tLb	; lb	Pound
	.db $7f,$45,$35,$92,$37,0,0,0,	$50,$51,$50,$50,$50,$50,$50

ounce:	.db 4,$bb,tLo,$bb,tLz	; oz	Ounce		= 1/16 lb
	.db $7e,$28,$34,$95,$23,0,0,0,	$50,$51,$50,$50,$50,$50,$50

dram:	.db 4,$bb,tLd,$bb,tLsmallr	; Dram		= 1/16 oz
	.db $7d,$17,$71,$84,$52,0,0,0,	$50,$51,$50,$50,$50,$50,$50

ton:	.db 6,$bb,tLsmallt,$bb,tLo,$bb,tLsmalln	; Ton	= 2000 lb
	.db $82,$90,$71,$84,$74,0,0,0,	$50,$51,$50,$50,$50,$50,$50

carat:	.db 4,$bb,tLc,$bb,tLsmallt ; ct	Carat
	.db $7f,$20,0,0,0,0,0,0,	$50,$51,$50,$50,$50,$50,$50

slug:	.db 8,$bb,tLs,$bb,tLl,$bb,tLu,$bb,tLsmallg ; Slug
	.db $81,$14,$59,$39,$03,0,0,0,	$50,$51,$50,$50,$50,$50,$50

	;; TIME

minute:	.db 6,$bb,tLm,$bb,tLi,$bb,tLsmalln ; Minute
	.db $81,$60,0,0,0,0,0,0,	$50,$50,$51,$50,$50,$50,$50

hour:	.db 4,$bb,tLh,$bb,tLsmallr	; Hour
	.db $83,$36,0,0,0,0,0,0,	$50,$50,$51,$50,$50,$50,$50

day:	.db 6,$bb,tLd,$bb,tLa,$bb,tLy	; Day
	.db $84,$86,$40,0,0,0,0,0,	$50,$50,$51,$50,$50,$50,$50

week:	.db 4,$bb,tLw,$bb,tLk	; wk	Week
	.db $85,$60,$48,0,0,0,0,0,	$50,$50,$51,$50,$50,$50,$50

month:	.db 4,$bb,tLm,$bb,tLo	; mo	Month (1/12 yr)
	.db $86,$26,$29,$74,$38,0,0,0,	$50,$50,$51,$50,$50,$50,$50

year:	.db 4,$bb,tLy,$bb,tLsmallr	; Year
	.db $87,$31,$55,$69,$25,$97,$46,$78, $50,$50,$51,$50,$50,$50,$50

hertz:	.db 3,'H',$bb,tLz	; Hz	Hertz
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$49,$50,$50,$50,$50

	;; TEMPERATURE

degR:	.db 2,tFromDeg,'R'	; degR	degree Rankine
	.db $7f,$55,$55,$55,$55,$55,$55,$55,$50,$50,$50,$50,$51,$50,$50

	;; AREA

are:	.db 6,$bb,tLa,$bb,tLsmallr,$bb,tLsmalle ; Are
	.db $82,$10,0,0,0,0,0,0,	$52,$50,$50,$50,$50,$50,$50

acre:	.db 8,$bb,tLa,$bb,tLc,$bb,tLsmallr,$bb,tLsmalle ; Acre
	.db $83,$40,$46,$85,$64,0,0,0,	$52,$50,$50,$50,$50,$50,$50

	;; VOLUME

liter:	.db 2,$bb,tLl		; l	Liter
	.db $7d,$10,0,0,0,0,0,0,	$53,$50,$50,$50,$50,$50,$50

cc:	.db 4,$bb,tLc,$bb,tLc	; cc	Cubic centimeter
	.db $7a,$10,0,0,0,0,0,0,	$53,$50,$50,$50,$50,$50,$50

gallon:	.db 6,$bb,tLsmallg,$bb,tLa,$bb,tLl ; Gallon
	.db $7d,$37,$85,$41,$18,0,0,0,	$53,$50,$50,$50,$50,$50,$50

quart:	.db 4,$bb,tLq,$bb,tLsmallt ; qt	Quart
	.db $7c,$94,$63,$52,$95,0,0,0,	$53,$50,$50,$50,$50,$50,$50

cup:	.db 6,$bb,tLc,$bb,tLu,$bb,tLp ; Cup
	.db $7c,$23,$65,$88,$24,0,0,0,	$53,$50,$50,$50,$50,$50,$50

pint:	.db 4,$bb,tLp,$bb,tLsmallt ; pt	Pint
	.db $7c,$47,$31,$76,$47,0,0,0,	$53,$50,$50,$50,$50,$50,$50

floz:	.db 8,$bb,tLf,$bb,tLl,$bb,tLo,$bb,tLz ; Fluid ounce
	.db $7b,$29,$57,$35,$30,0,0,0,	$53,$50,$50,$50,$50,$50,$50

barrel:	.db 6,$bb,tLb,$bb,tLb,$bb,tLl ; Barrel
	.db $7f,$15,$89,$87,$29,0,0,0,	$53,$50,$50,$50,$50,$50,$50

bushel:	.db 4,$bb,tLb,$bb,tLu	; bu	Bushel
	.db $7e,$35,$23,$90,$70,0,0,0,	$53,$50,$50,$50,$50,$50,$50

peck:	.db 4,$bb,tLp,$bb,tLk	; pk	Peck
	.db $7d,$88,$09,$76,$75,0,0,0,	$53,$50,$50,$50,$50,$50,$50

tbsp:	.db 8,$bb,tLsmallt,$bb,tLb,$bb,tLs,$bb,tLp
	.db $7b,$14,$78,$67,$65,0,0,0,	$53,$50,$50,$50,$50,$50,$50

tsp:	.db 6,$bb,tLsmallt,$bb,tLs,$bb,tLp
	.db $7a,$49,$28,$92,$16,0,0,0,	$53,$50,$50,$50,$50,$50,$50

ccf:	.db 6,$bb,tLc,$bb,tLc,$bb,tLf	; ccf		= 100 ft^3
	.db $80,$28,$31,$68,$47,0,0,0,	$53,$50,$50,$50,$50,$50,$50

Mcf:	.db 5,'M',$bb,tLc,$bb,tLf	; Mcf		= 1000 ft^3
	.db $81,$28,$31,$68,$47,0,0,0,	$53,$50,$50,$50,$50,$50,$50


	;; VELOCITY

knot:	.db 8,$bb,tLk,$bb,tLsmalln,$bb,tLo,$bb,tLsmallt
	.db $7f,$51,$44,$44,$44,$44,$44,$44,$51,$50,$49,$50,$50,$50,$50

mph:	.db 6,$bb,tLm,$bb,tLp,$bb,tLh ; mph		= mi/hr
	.db $7f,$44,$70,$40,0,0,0,0,	$51,$50,$49,$50,$50,$50,$50

kph:	.db 6,$bb,tLk,$bb,tLp,$bb,tLh ; kph		= km/hr
	.db $7f,$27,$77,$77,$77,$77,$77,$77,$51,$50,$49,$50,$50,$50,$50

fpm:	.db 6,$bb,tLf,$bb,tLp,$bb,tLm ; fpm		= ft/min
	.db $7d,$50,$80,0,0,0,0,0,	$51,$50,$49,$50,$50,$50,$50

fps:	.db 6,$bb,tLf,$bb,tLp,$bb,tLs ; fps		= ft/sec
	.db $7f,$30,$48,0,0,0,0,0,	$51,$50,$49,$50,$50,$50,$50

ipy:	.db 6,$bb,tLi,$bb,tLp,$bb,tLy	; ipy		= in/yr
	.db $76,$80,$48,$94,$62,0,0,0,	$51,$50,$49,$50,$50,$50,$50


	;; ACCELERATION

gal:	.db 5,'G',$bb,tLa,$bb,tLl ; Gal(ileo)
	.db $7e,$10,0,0,0,0,0,0,	$51,$50,$48,$50,$50,$50,$50

	;; DENSITY

gsm:	.db 6,$bb,tLsmallg,$bb,tLs,$bb,tLm ; grams per square meter
	.db $7d,$10,0,0,0,0,0,0,	$48,$51,$50,$50,$50,$50,$50

	;; FORCE

gf:	.db 4,$bb,tLsmallg,$bb,tLf ; gf	Gram "force"
	.db $7d,$98,$06,$65,0,0,0,0,	$51,$51,$48,$50,$50,$50,$50

lbf:	.db 6,$bb,tLl,$bb,tLb,$bb,tLf ; Pound force
	.db $80,$44,$48,$22,$16,0,0,0,	$51,$51,$48,$50,$50,$50,$50

dyne:	.db 8,$bb,tLd,$bb,tLy,$bb,tLsmalln,$bb,tLsmalle
	.db $7b,$10,0,0,0,0,0,0,	$51,$51,$48,$50,$50,$50,$50

pond:	.db 2,$bb,tLp		; p	Pond		= gram * g
	.db $7d,$98,$06,$65,0,0,0,0,	$51,$51,$48,$50,$50,$50,$50


	;; PRESSURE

atm:	.db 6,$bb,tLa,$bb,tLsmallt,$bb,tLm ; atm
	.db $85,$10,$13,$25,0,0,0,0,	$49,$51,$48,$50,$50,$50,$50

bar:	.db 6,$bb,tLb,$bb,tLa,$bb,tLsmallr ; bar
	.db $85,$10,0,0,0,0,0,0,	$49,$51,$48,$50,$50,$50,$50

vac:	.db 6,$bb,tLv,$bb,tLa,$bb,tLc ; vac
	.db $82,$10,0,0,0,0,0,0,	$49,$51,$48,$50,$50,$50,$50

psi:	.db 6,$bb,tLp,$bb,tLs,$bb,tLi ; psi
	.db $83,$68,$94,$75,$73,0,0,0,	$49,$51,$48,$50,$50,$50,$50

mercur:	.db 3,'H',$bb,tLsmallg	; Hg	Standard weight of mercury
	.db $85,$13,$33,$22,$39,0,0,0,	$48,$51,$48,$50,$50,$50,$50

water:	.db 3,"H2O"		; H20	Standard weight of water
	.db $83,$98,$06,$65,0,0,0,0,	$48,$51,$48,$50,$50,$50,$50

mhg:	.db 5,$bb,tLm,'H',$bb,tLsmallg ; mHg	Meter mercury
	.db $85,$13,$33,$22,$39,0,0,0,	$49,$51,$48,$50,$50,$50,$50

mh2o:	.db 5,$bb,tLm,"H2O"	; mH2O	Meter water
	.db $83,$98,$06,$65,0,0,0,0,	$49,$51,$48,$50,$50,$50,$50

inhg:	.db 7,$bb,tLi,$bb,tLn,'H',$bb,tLsmallg
	.db $83,$33,$86,$38,$86,0,0,0,	$49,$51,$48,$50,$50,$50,$50

inh2o:	.db 7,$bb,tLi,$bb,tLn,"H2O"
	.db $82,$24,$90,$88,$91,0,0,0,	$49,$51,$48,$50,$50,$50,$50

torr:	.db 8,$bb,tLsmallt,$bb,tLo,$bb,tLsmallr,$bb,tLsmallr
	.db $82,$13,$33,$22,$39,0,0,0,	$49,$51,$48,$50,$50,$50,$50

	;; VISCOSITY

poise:	.db 1,'P'		; = gram / cm s
	.db $7f,$10,0,0,0,0,0,0,	$49,$51,$49,$50,$50,$50,$50

	;; KINEMATIC VISCOSITY

stokes:	.db 3,'S',$bb,tLsmallt	; = cm^2 / s
	.db $7c,$10,0,0,0,0,0,0,	$52,$50,$49,$50,$50,$50,$50

	;; ENERGY

calor:	.db 6,$bb,tLc,$bb,tLa,$bb,tLl ;	calorie
	.db $80,$41,$86,$80,0,0,0,0,	$52,$51,$48,$50,$50,$50,$50
	;; Sufficient energy to raise the temperature of 1 g of water
	;; by 1 degC.

Calor:	.db 5,'C',$bb,tLa,$bb,tLl ;	*C*alorie
	.db $83,$41,$86,$80,0,0,0,0,	$52,$51,$48,$50,$50,$50,$50

eV:	.db 3,$bb,tLsmalle,'V'	; eV	electron-volt
	.db $6d,$16,$02,$17,$65,0,0,0,	$52,$51,$48,$50,$50,$50,$50

erg:	.db 6,$bb,tLsmalle,$bb,tLsmallr,$bb,tLsmallg
	.db $79,$10,0,0,0,0,0,0,	$52,$51,$48,$50,$50,$50,$50

btu:	.db 6,$bb,tLb,$bb,tLsmallt,$bb,tLu	; British thermal unit
	.db $83,$10,$55,$05,$59,0,0,0,	$52,$51,$48,$50,$50,$50,$50

chu:	.db 6,$bb,tLc,$bb,tLh,$bb,tLu		; Celsius heat unit
	.db $83,$18,$99,$10,$05,0,0,0,	$52,$51,$48,$50,$50,$50,$50

toe:	.db 6,$bb,tLsmallt,$bb,tLo,$bb,tLsmalle	; Ton oil equivalent
	.db $8a,$41,$86,$80,0,0,0,0,	$52,$51,$48,$50,$50,$50,$50

tnt:	.db 3,"TNT"		; TNT	energy per kilogram
	.db $86,$46,$12,$07,$05,0,0,0,	$52,$50,$48,$50,$50,$50,$50

Wh:	.db 3,'W',$bb,tLh	; Wh	Watt hour
	.db $83,$36,0,0,0,0,0,0,	$52,$51,$48,$50,$50,$50,$50

hph:	.db 6,$bb,tLh,$bb,tLp,$bb,tLh	; hph Horsepower hour
	.db $86,$26,$84,$51,$95,0,0,0,	$52,$51,$48,$50,$50,$50,$50


	;; POWER

horse:	.db 4,$bb,tLh,$bb,tLp	; hp	Horsepower
	.db $82,$74,$56,$99,$87,0,0,0,	$52,$51,$47,$50,$50,$50,$50

mbh:	.db 6,$bb,tLm,$bb,tLb,$bb,tLh	; mbh		= 1000 btu/hr
	.db $82,$29,$30,$71,$07,0,0,0,	$52,$51,$47,$50,$50,$50,$50


	;; ASSORTED PHOTOMETRIC UNITS

lumen:	.db 4,$bb,tLl,$bb,tLm	; lm	Lumen
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$51

lux:	.db 4,$bb,tLl,$bb,tLx	; lx	Lux
	.db $80,$10,0,0,0,0,0,0,	$48,$50,$50,$50,$50,$50,$51

phot:	.db 4,$bb,tLp,$bb,tLh	; ph	Phot
	.db $84,$10,0,0,0,0,0,0,	$48,$50,$50,$50,$50,$50,$51

mcs:	.db 6,$bb,tLm,$bb,tLc,$bb,tLs ;	mcs = meter candle second
	.db $80,$10,0,0,0,0,0,0,	$48,$50,$51,$50,$50,$50,$51

lam:	.db 6,$bb,tLl,$bb,tLa,$bb,tLm
	.db $83,$31,$83,$09,$89,0,0,0,	$48,$50,$50,$50,$50,$50,$51

nit:	.db 6,$bb,tLsmalln,$bb,tLi,$bb,tLsmallt
	.db $80,$10,0,0,0,0,0,0,	$48,$50,$50,$50,$50,$50,$51

sb:	.db 4,$bb,tLs,$bb,tLb
	.db $84,$10,0,0,0,0,0,0,	$48,$50,$50,$50,$50,$50,$51

asb:	.db 6,$bb,tLa,$bb,tLs,$bb,tLb
	.db $7f,$31,$83,$09,$89,0,0,0,	$48,$50,$50,$50,$50,$50,$51

fl:	.db 3,$bb,tLf,'L'
	.db $80,$34,$26,$25,$91,0,0,0,	$48,$50,$50,$50,$50,$50,$51

	;; ASSORTED RADIATION UNITS

Bq:	.db 3,'B',$bb,tLq	; Bq	Becquerel
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$49,$50,$50,$50,$50

Ci:	.db 3,'C',$bb,tLi	; Ci	Curie
	.db $8a,$37,0,0,0,0,0,0,	$50,$50,$49,$50,$50,$50,$50

Gy:	.db 3,'G',$bb,tLy	; Gy	Gray
	.db $80,$10,0,0,0,0,0,0,	$52,$50,$48,$50,$50,$50,$50

rad:	.db 6,$bb,tLsmallr,$bb,tLa,$bb,tLd ; Radiation absorbed dose
	.db $7e,$10,0,0,0,0,0,0,	$52,$50,$48,$50,$50,$50,$50

rep:	.db 6,$bb,tLsmallr,$bb,tLsmalle,$bb,tLp	; Roentgen equiv physical
	.db $7d,$83,$80,0,0,0,0,0,	$52,$50,$48,$50,$50,$50,$50

Sv:	.db 3,'S',$bb,tLv	; Sv	Sievert
	.db $80,$10,0,0,0,0,0,0,	$52,$50,$48,$50,$50,$50,$50

rem:	.db 6,$bb,tLsmallr,$bb,tLsmalle,$bb,tLm ; Roentgen equiv mammal
	.db $7e,$10,0,0,0,0,0,0,	$52,$50,$48,$50,$50,$50,$50


	;; FLUID FLOW

gph:	.db 6,$bb,tLsmallg,$bb,tLp,$bb,tLh ; gal/hr
	.db $7a,$10,$51,$50,$33,0,0,0,	$53,$50,$49,$50,$50,$50,$50

gpm:	.db 6,$bb,tLsmallg,$bb,tLp,$bb,tLm ; gal/min
	.db $7b,$63,$09,$01,$96,0,0,0,	$53,$50,$49,$50,$50,$50,$50

mgd:	.db 6,$bb,tLm,$bb,tLsmallg,$bb,tLd ; megagal/day
	.db $7e,$43,$81,$26,$36,0,0,0,	$53,$50,$49,$50,$50,$50,$50

cfs:	.db 6,$bb,tLc,$bb,tLf,$bb,tLs	; ft^3/sec
	.db $7a,$28,$31,$68,$47,0,0,0,	$53,$50,$49,$50,$50,$50,$50

cfh:	.db 6,$bb,tLc,$bb,tLf,$bb,tLh	; ft^3/hr
	.db $7a,$10,$51,$50,$33,0,0,0,	$53,$50,$49,$50,$50,$50,$50

cfm:	.db 6,$bb,tLc,$bb,tLf,$bb,tLm	; ft^3/min
	.db $7c,$47,$19,$47,$44,0,0,0,	$53,$50,$49,$50,$50,$50,$50

lpm:	.db 6,$bb,tLl,$bb,tLp,$bb,tLm	; L/min
	.db $7b,$16,$66,$66,$66,$66,$66,$66,$53,$50,$49,$50,$50,$50,$50

lfm:	.db 6,$bb,tLl,$bb,tLf,$bb,tLm	; ft/min
	.db $7d,$50,$80,0,0,0,0,0,	$51,$50,$49,$50,$50,$50,$50

	;; GAS FLOW

sccm:	.db 8,$bb,tLs,$bb,tLc,$bb,tLc,$bb,tLm	; Standard cc/min
	.db $7d,$16,$88,$75,0,0,0,0,	$52,$51,$47,$50,$50,$50,$50

sccs:	.db 8,$bb,tLs,$bb,tLc,$bb,tLc,$bb,tLs	; Standard cc/sec
	.db $7f,$10,$13,$25,0,0,0,0,	$52,$51,$47,$50,$50,$50,$50

scfh:	.db 8,$bb,tLs,$bb,tLc,$bb,tLf,$bb,tLh	; Standard ft^3/hr
	.db $7f,$79,$70,$01,$24,0,0,0,	$52,$51,$47,$50,$50,$50,$50

scfm:	.db 8,$bb,tLs,$bb,tLc,$bb,tLf,$bb,tLm	; Standard ft^3/min
	.db $81,$47,$82,$00,$75,0,0,0,	$52,$51,$47,$50,$50,$50,$50

slph:	.db 8,$bb,tLs,$bb,tLl,$bb,tLp,$bb,tLh	; Standard liter/hr
	.db $7e,$28,$14,$58,$33,0,0,0,	$52,$51,$47,$50,$50,$50,$50

slpm:	.db 8,$bb,tLs,$bb,tLl,$bb,tLp,$bb,tLm	; Standard liter/min
	.db $7e,$28,$14,$58,$33,0,0,0,	$52,$51,$47,$50,$50,$50,$50

lusec:	.db 10,$bb,tLl,$bb,tLu,$bb,tLs,$bb,tLsmalle,$bb,tLc
	.db $7c,$13,$33,$22,$39,0,0,0,	$52,$51,$47,$50,$50,$50,$50

	;; Miscellany

mpg:	.db 6,$bb,tLm,$bb,tLp,$bb,tLsmallg ; mpg	= mi/gal
	.db $85,$42,$51,$43,$71,0,0,0,	$48,$50,$50,$50,$50,$50,$50



UnitStrings_WRITE:		; NO DIMENSIONLESS UNITS BELOW THIS POINT!

;;; Automatically preferred (These may not be in the right order.)
;;; (>DEFAULT displays these.)

henry:	.db 1,'H'		; H	Henry
	.db $80,$10,0,0,0,0,0,0,	$52,$51,$48,$48,$50,$50,$50

tesla:	.db 1,'T'		; T	Tesla
	.db $80,$10,0,0,0,0,0,0,	$50,$51,$48,$49,$50,$50,$50

weber:	.db 3,'W',$bb,tLb	; Wb	Weber
	.db $80,$10,0,0,0,0,0,0,	$52,$51,$48,$49,$50,$50,$50

farad:	.db 1,'F'		; F	Farad
	.db $80,$10,0,0,0,0,0,0,	$48,$49,$54,$52,$50,$50,$50

ohm:	.db 2,$bb,tLcapOmega	; (Om)	Ohm
	.db $80,$10,0,0,0,0,0,0,	$52,$51,$47,$48,$50,$50,$50

siemen:	.db 1,'S'		; S	Siemens
	.db $80,$10,0,0,0,0,0,0,	$48,$49,$53,$52,$50,$50,$50

volt:	.db 1,'V'		; W	Volt
	.db $80,$10,0,0,0,0,0,0,	$52,$51,$47,$49,$50,$50,$50

watt:	.db 1,'W'		; W	Watt
	.db $80,$10,0,0,0,0,0,0,	$52,$51,$47,$50,$50,$50,$50

joule:	.db 1,'J'		; J	Joule
	.db $80,$10,0,0,0,0,0,0,	$52,$51,$48,$50,$50,$50,$50

pascal:	.db 3,'P',$bb,tLa	; Pa	Pascal
	.db $80,$10,0,0,0,0,0,0,	$49,$51,$48,$50,$50,$50,$50

newton:	.db 1,'N'		; N	Newton
	.db $80,$10,0,0,0,0,0,0,	$51,$51,$48,$50,$50,$50,$50

coulom:	.db 1,'C'		; C	Coulomb
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$51,$51,$50,$50,$50

;;; Base units

meter:	.db 2,$bb,tLm		; m	Meter
	.db $80,$10,0,0,0,0,0,0,	$51,$50,$50,$50,$50,$50,$50
	;; Length of the path traveled by light in a vacuum
	;; during 1/299792458 seconds.

gram:	.db 2,$bb,tLsmallg	; g	Gram
	.db $7d,$10,0,0,0,0,0,0,	$50,$51,$50,$50,$50,$50,$50
	;; 1/1000 mass of the international prototype

second:	.db 2,$bb,tLs		; s	Second
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$51,$50,$50,$50,$50
	;; Duration of 9192631770 periods of the radiation
	;; corresponding to the transition between the two hyperfine
        ;; levels of the ground state of the cesium-133 atom

ampere:	.db 1,'A'		; A	Ampere
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$50,$51,$50,$50,$50
	;; The current which produces a force of 2e-7 N/m between two
	;; infinitely long wires that are 1 meter apart

kelvin:	.db 1,'K'		; K	Kelvin
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$51,$50,$50
	;; 1/273.16 of the thermodynamic temperature of the triple
	;; point of water

mole:	.db 6,$bb,tLm,$bb,tLo,$bb,tLl ;	Mole
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$51,$50
	;; The amount of substance of a system which contains as many
	;; elementary entities as there are atoms in 0.012 kg of
        ;; carbon 12.

candel:	.db 3,'C',$bb,$tLd	; Cd	Candela
	.db $80,$10,0,0,0,0,0,0,	$50,$50,$50,$50,$50,$50,$51
	;; Luminous intensity in a given direction of a source which
	;; emits monochromatic radiation at 540e9 Hz with radiant
	;; intensity 1|683 W/steradian.


	.db 0

