;;; -*- mode: TI-Asm; ti-asm-listing-file: "units.lst" -*-
;;;
;;;	Unit Operations Flash App for TI-83+
;;; 
;;;	Copyright 2003 Benjamin Moody
;;;
;;;  This program is free software; you can redistribute
;;;  and/or modify it under the terms of the GNU General
;;;  Public License as published by the Free Software
;;;  Foundation; either version 2 of the License, or (at
;;;  your option) any later version.
;;;
;;;  This program is distributed in the hope that it will be
;;;  useful, but WITHOUT ANY WARRANTY; without even the
;;;  implied warranty of MERCHANTABILITY or FITNESS FOR A
;;;  PARTICULAR PURPOSE. See the GNU General Public License
;;;  for more details.
;;;
;;;  You should have received a copy of the GNU General
;;;  Public License along with this program; if not, write
;;;  to the Free Software Foundation, Inc., 59 Temple Place
;;;  - Suite 330, Boston, MA 02111 USA.
;;;

;;; units.asm: Main program code

; Include files omitted from listing
.nolist
#include "ti83plus.inc"
#include "extraequ.inc"
#include "hooks.inc"
#include "dstoks.inc"
.list

;;; Display whatever is in OP1
_FormDisp = $497e

;;; Fetch the next byte from program (C flag for EOF)
_IncFetch = $4B73
;;; Pointers used by the above
parseCur = $965D
parseTop = $965B
parseBot = $965F

;;; Create a temporary variable
_CreateTemp = $4A4D

#define rOP1ToOP2 08h
#define rFindSym 10h
#define rPushRealO1 18h
#define rMov9ToOP1 20h
#define rFPAdd 30h



VMajor = 0
VMinor = 8

	.org $4000

	.db $80, $0F		; Field: Program length
	.db 0, 0, 0, 0
	.db $80, $12		; Field: Program type
	.db $01, $04
	.db $80, $21		; Field: App ID
	.db VMajor+1
	.db $80, $31		; Field: App Build
	.db VMinor+1
	.db $80, $48		; Field: App Name
	.db "Unit Ops"
	.db $80, $81		; Field: App Pages
	.db 1
	.db $80, $90		; No default splash screen
	.db $03, $26, $09, $04, $04, $6F, $1B, $80	; Date stamp- 5/12/1999
	.db $02, $0D, $40, $A1, $6B, $99, $F6, $59, $BC, $67	; Dummy
	.db $F5, $85, $9C, $09, $6C, $0F, $B4, $03, $9B, $C9	; encrypted TI
	.db $03, $32, $2C, $E0, $03, $20, $E3, $2C, $F4, $2D	; date stamp
	.db $73, $B4, $27, $C4, $A0, $72, $54, $B9, $EA, $7C	; signature
	.db $3B, $AA, $16, $F6, $77, $83, $7A, $EE, $1A, $D4
	.db $42, $4C, $6B, $8B, $13, $1F, $BB, $93, $8B, $FC 
	.db $19, $1C, $3C, $EC, $4D, $E5, $75
	.db $80, $7F		; Field: Program Image length
	.db   0, 0, 0, 0
	.db $80,$A1		; Field: Hardware Compatibility Mode
	.db $FF

	.db 0,0,0,0
	.db 0,0,0,0
	.db 0,0,0,0
	.db 0

;;; Temporary storage locations (must be parserhook-safe!)

;;; Actually, many of these locations are used in both format and parse,
;;; but I'm too lazy to label them

unitPrefix	= appBackUpScreen	; Exponent of prefix
appVarPtr	= appBackUpScreen+1	; Pointer to start of prefs
lastPrefixPtr	= appBackUpScreen+3	; Pointer to prefix entry
helpPtr		= lastPrefixPtr		; Pointer to current help page
tempUnitPtr	= appBackUpScreen+5	; Pointer to unit entry
formatPtr	= appBackUpScreen+7	; Pointer to end of format string
parseNumDen	= appBackUpScreen+9	; Based on the previous separator
currentExponent = appBackUpScreen+10	; Exponent of a complete unit
tempStringMem	= appBackUpScreen+11
OP7		= appBackUpScreen+72	; Scratch space, used by exp and read
OP8		= appBackUpScreen+83	; Used by read
OP9		= appBackUpScreen+94	; Used by read
OP10		= appBackUpScreen+105	; Used by exp
OP11		= appBackUpScreen+116	; Used by exp
backupHooks	= appBackUpScreen+127	; Backups of shared hooks
preparseChangedP = appBackUpScreen+136
customAppVar	= appBackUpScreen+137
customAVPage	= appBackUpScreen+139
customUnit	= appBackUpScreen+140
formatString	= appBackUpScreen+172	; Format string

;;; These locations must be rawkeyhook-safe
appVarSave	= saveSScreen
menuX		= saveSScreen+2
menuY		= saveSScreen+3
menuZ		= saveSScreen+4
menuSub		= saveSScreen+5
menuPtr		= saveSScreen+6
menuBot		= saveSScreen+8
menuPrefix	= saveSScreen+9
tempLoadingOPs	= saveSScreen+10	; Used to save OPs during avar load

;;; Units are ALWAYS STORED IN SI BASE UNITS.
;;; There are seven of them, which is convenient for the 7 mantissa bytes...
;;; The order: meter, kilogram, second, ampere, kelvin, mole, candela


;;; The whole hook issue is getting quite ugly. There's a more-or-less standard
;;; way of handling multiple hooks on the 68k's; we seriously need to work out
;;; something similar...

	jp StartOApp
	.db VMajor,VMinor
	.dw ParserHook
	.dw CxM_HomeHook
	.dw RawkeyHook
	.dw PrefixStrings
	.dw UnitStrings
	.dw ParserTable

StartOApp:
	B_CALL _ClrLCDFull
	ld hl,0
	ld (curCol),hl

	ld hl,appVarName
	rst 20h
	B_CALL _ChkFindSym
	jr c,StartupOK
	ld a,b
	or a
	ld hl,StartupArcStr
	jr nz,StartupError
	inc de
	inc de
	ld hl,appVarSig
	ld b,appVarSigLen
	call CompStrsN
	jr z,StartupOK
	ld hl,StartupInvalStr
StartupError:
	push hl
	ld hl,StartupErrorStr
	call PutS
	pop hl
	call PutS
	B_CALL _GetKey
	B_CALL _JForceCmdNoChar

StartupOK:
	set lwrCaseActive,(iy+appLwrCaseFlag)
	set textInverse,(iy+textFlags)
	ld hl,0
	ld de,$135E
	B_CALL _FillRect
	set fracDrawLFont,(iy+fontFlags)
	ld hl,WelcomeStr
	ld a,2
	jr TitleDraw
TitleDrawNL:
	ld a,11
TitleDraw:
	ld (penRow),a
	ld a,3
	ld (penCol),a
TitleDrawLoop:
	ld a,(hl)
	or a
	jr z,TitleDrawDone
	B_CALL _VPutMap
	jr c,TitleDrawNL
	inc hl
	jr TitleDrawLoop
TitleDrawDone:
	res fracDrawLFont,(iy+fontFlags)
	res textInverse,(iy+textFlags)
MainMenu:
	call ClearMenu
	ld b,5
	ld hl,MainMenuStrs
	ld c,'1'
MainMenuDrawLoop:
	push bc
	ld a,c
	B_CALL _PutC
	ld a,':'
	B_CALL _PutC
	call PutS
	pop bc
	inc c
	djnz MainMenuDrawLoop
MainMenuKeyLoop:
	res onInterrupt,(iy+onFlags)
	B_CALL _GetKey
	cp kQuit
	jr z,Quit
	cp kClear
	jr z,Quit
	sub k1
	jr c,MainMenuKeyLoop
	jr z,Install
	dec a
	jr z,Uninstall
	dec a
	jr z,Advanced
	dec a
	jp z,Help
	dec a
	jr nz,MainMenuKeyLoop
Quit:	B_CALL _JForceCmdNoChar

Install:
	call ClearMenu
	call LoadAppVar		; create if doesn't already exist
	call InstallParserHook
	jr c,Cancel
	call InstallCxmHomeHook
	jr c,Cancel
	call InstallRawKeyHook
	jr c,Cancel
	ld hl,InstalledStr
	call PutS
	B_CALL _GetKey
	jr MainMenu
Cancel:
	ld hl,CancelStr
	call PutS
	B_CALL _GetKey
	jr MainMenu

Uninstall:
	call LoadAppVar
	ld (appVarPtr),de
	ld de,OFF_PARSER	
	ld hl,parserHookPtr
	call UninstallHook
	jr nz,Uninstall_NotP
	res parserHookActive,(iy+parserHookFlag)
Uninstall_NotP:
	ld de,OFF_HOMEHOOK
	ld hl,cxM_HomeHookPtr
	call UninstallHook
	jr nz,Uninstall_NotH
	res cxM_HomeHookActive,(iy+cxM_HomeHookFlag)
Uninstall_NotH:
	ld de,OFF_RAWKEY
	ld hl,rawKeyHookPtr
	call UninstallHook
	jr nz,Uninstall_NotR
	res rawKeyHookActive,(iy+rawKeyHookFlag)
Uninstall_NotR:
	call ClearMenu
	ld hl,UninstalledStr
	call PutS
	B_CALL _GetKey
	jp MainMenu

Advanced:
	call ClearMenu
	ld b,5
	ld hl,AdvancedStrs
	ld c,'1'
AdvancedDrawLoop:
	push bc
	ld a,c
	B_CALL _PutC
	ld a,':'
	B_CALL _PutC
	call PutS
	B_CALL _NewLine
	pop bc
	inc c
	djnz AdvancedDrawLoop

	ld hl,10*256+3
	ld (curRow),hl
	ld hl,OffStr
	bit parserHookActive,(iy+parserHookFlag)
	jr z,AdvancedPHookStr
	in a,(6)
	ld b,a
	ld a,(parserHookPtr+2)
	cp b
	ld hl,A_InUseStr
	jr nz,AdvancedPHookStr
	call LoadAppVar
	ld (appVarPtr),de
	ld hl,OFF_PARSER+2
	add hl,de
	ld a,(hl)
	or a
	ld hl,OnStr
	jr z,AdvancedPHookStr
	ld hl,SharedStr
AdvancedPHookStr:
	call PutS

	ld hl,10*256+4
	ld (curRow),hl
	ld hl,OffStr
	bit cxM_HomeHookActive,(iy+cxM_HomeHookFlag)
	jr z,AdvancedHHookStr
	in a,(6)
	ld b,a
	ld a,(cxM_HomeHookPtr+2)
	cp b
	ld hl,A_InUseStr
	jr nz,AdvancedHHookStr
	ld hl,(appVarPtr)
	ld de,OFF_HOMEHOOK+2
	add hl,de
	ld a,(hl)
	or a
	ld hl,OnStr
	jr z,AdvancedHHookStr
	ld hl,SharedStr
AdvancedHHookStr:
	call PutS

	ld hl,10*256+5
	ld (curRow),hl
	ld hl,OffStr
	bit rawKeyHookActive,(iy+rawKeyHookFlag)
	jr z,AdvancedRHookStr
	in a,(6)
	ld b,a
	ld a,(rawKeyHookPtr+2)
	cp b
	ld hl,A_InUseStr
	jr nz,AdvancedRHookStr
	ld hl,(appVarPtr)
	ld de,OFF_RAWKEY+2
	add hl,de
	ld a,(hl)
	or a
	ld hl,OnStr
	jr z,AdvancedRHookStr
	ld hl,SharedStr
AdvancedRHookStr:
	call PutS

AdvancedKeyLoop:
	res onInterrupt,(iy+onFlags)
	B_CALL _GetKey
	cp kQuit
	jp z,Quit
	cp kClear
	jp z,MainMenu
	sub k1
	jr c,AdvancedKeyLoop
	jr z,AParser
	dec a
	jr z,ACxmHome
	dec a
	jr z,ARawKey
	dec a
 	jp z,AHelp
	dec a
	jr nz,AdvancedKeyLoop
	jp MainMenu

AParser:
	call ClearMenu
	bit parserHookActive,(iy+parserHookFlag)
	jr z,AParserInstall
	ld hl,parserHookPtr+2
	ld a,(hl)
	ld b,a
	in a,(6)
	cp b
	jr nz,AParserInstall
	ld de,OFF_PARSER
	dec hl
	dec hl
	call UninstallHook
	jr nz,Advanced_Frog0
	res parserHookActive,(iy+parserHookFlag)
Advanced_Frog0:
	jp Advanced
AParserInstall:
	call InstallParserHook
	jr Advanced_Frog0

ACxmHome:
	call ClearMenu
	bit cxM_HomeHookActive,(iy+cxM_HomeHookFlag)
	jr z,ACxmHomeInstall
	ld a,(cxM_HomeHookPtr+2)
	ld b,a
	in a,(6)
	cp b
	jr nz,ACxmHomeInstall
	ld de,OFF_HOMEHOOK
	ld hl,cxM_HomeHookPtr
	call UninstallHook
	jr nz,Advanced_Frog0
	res cxM_HomeHookActive,(iy+cxM_HomeHookFlag)
	jr Advanced_Frog0
ACxmHomeInstall:
	call InstallCxmHomeHook
	jr Advanced_Frog0

ARawKey:
	call ClearMenu
	bit rawKeyHookActive,(iy+rawKeyHookFlag)
	jr z,ARawKeyInstall
	ld a,(rawKeyHookPtr+2)
	ld b,a
	in a,(6)
	cp b
	jr nz,ARawKeyInstall
	ld de,OFF_RAWKEY
	ld hl,rawKeyHookPtr
	call UninstallHook
	jr nz,Advanced_Frog0
	res rawKeyHookActive,(iy+rawKeyHookFlag)
	jr Advanced_Frog0
ARawKeyInstall:
	call InstallRawKeyHook
	jr Advanced_Frog0

UninstallHook:
	push hl
	ld hl,(appVarPtr)
	add hl,de
	pop de
	push hl
	call Mov3
	dec de
	ld a,(de)
	pop de
	ld hl,Three0s
	call Mov3
	or a
	ret



InstallParserHook:
	bit parserHookActive,(iy+parserHookFlag)
	ld hl,parserHookPtr+2
	ld de,ParserStr
	ld bc,OFF_PARSER
	call CheckHook
	ret nz
	in a,(6)
	ld hl,ParserHook
	B_CALL _EnableParserHook
	ret
InstallCxmHomeHook:
	bit cxM_HomeHookActive,(iy+cxM_HomeHookFlag)
	ld hl,cxM_HomeHookPtr+2
	ld de,CxM_HomeStr
	ld bc,OFF_HOMEHOOK
	call CheckHook
	ret nz
	in a,(6)
	ld hl,CxM_HomeHook
	B_CALL _EnableCxM_HomeHook
	ret
InstallRawKeyHook:
	bit rawKeyHookActive,(iy+rawKeyHookFlag)
	ld hl,rawKeyHookPtr+2
	ld de,RawKeyStr
	ld bc,OFF_RAWKEY
	call CheckHook
	ret nz
	in a,(6)
	ld hl,RawkeyHook
	B_CALL _EnableRawKeyHook
	ret

AHelp:	ld hl,AHelpText
	jr HelpAt
Help:	ld hl,HelpText
HelpAt:	ld (helpPtr),hl
HelpLoop:
	call ClearMenu
	ld hl,(helpPtr)
	call PutS
	ld a,(hl)
	or a
	jr z,HelpNoDown
	ld hl,15*256+7
	ld (curRow),hl
	ld a,$1f
	B_CALL _PutMap
HelpNoDown:
	ld hl,(helpPtr)
	dec hl
	dec hl
	ld a,(hl)
	or a
	jr z,HelpNoUp
	ld hl,15*256+3
	ld (curRow),hl
	ld a,$1e
	B_CALL _PutMap
HelpNoUp:
HelpKeyLoop:
	B_CALL _GetKey
	cp kDown
	jr z,HelpDown
	cp kUp
	jr z,HelpUp
	cp kQuit
	jp z,Quit
	cp kClear
	jp z,HelpQuit
	jr HelpKeyLoop
HelpDown:
	ld hl,(helpPtr)
HelpDownLoop:
	ld a,(hl)
	inc hl
	or a
	jr nz,HelpDownLoop
	ld a,(hl)
	or a
	jr z,HelpKeyLoop
	ld (helpPtr),hl
	jr HelpLoop
HelpUp:
	ld hl,(helpPtr)
	dec hl
	dec hl
	ld a,(hl)
	or a
	jr z,HelpKeyLoop
HelpUpLoop:
	dec hl
	ld a,(hl)
	or a
	jr nz,HelpUpLoop
	inc hl
	ld (helpPtr),hl
	jr HelpLoop
HelpQuit:
	ld hl,(helpPtr)
	ld de,-AHelpText
	add hl,de
	jp nc,MainMenu
	jp Advanced
	

CheckHook:
	ret z
	in a,(6)
	cp (hl)
	ret z
	push hl
	ex de,hl
	call PutS
	ld hl,InUseStr
	call PutS
	pop hl
CheckHookLoop:
	res onInterrupt,(iy+onFlags)
	push bc
	push hl
	B_CALL _GetKey
	pop hl
	pop bc
	sub k1
	jr c,CheckHookLoop
	jr z,CheckHookOK
	dec a
	jr z,CheckHookShare
	dec a
	jr z,CheckHookSkip
	dec a
	jr z,CheckHookCancel
	jr CheckHookLoop
CheckHookShare:
	push hl
	push bc
	call LoadAppVar
	pop hl
	add hl,de
	ex de,hl
	pop hl
	dec hl
	dec hl
	call Mov3
	xor a
	jr CheckHookOK
CheckHookCancel:
	scf
CheckHookSkip:
	inc a
CheckHookOK:
	push af
	call ClearMenu
	pop af
	ret

ClearMenu:
	ld hl,3
	ld (curRow),hl
	ld b,(16*5)
ClearMenuLoop:
	ld a,' '
	B_CALL _PutC
	djnz ClearMenuLoop
	ld hl,3
	ld (curRow),hl
	ret

AppVarVATName:
	.db "edoMtinU",8

LoadAppVar:
	call LoadAppVar_Quick
	ret z
	call Mov66OPTemp
	call LoadAppVar_Main
	push de
	call Mov66TempOP
	pop de
	ret

LoadAppVar_Create:
	ld hl,appVarDefLen
	push hl
	B_CALL _CreateAppVar
	inc de
	inc de
	ld hl,appVarDef
	pop bc
	ldir
	jr LoadAppVar
LoadAppVar_Unarch:
	B_CALL _Arc_Unarc
LoadAppVar_Main:
	;; Our app var is used to store preferred units.  In the future, it
	;; may also be used to store custom unit definitions as well as hook
	;; queues.
	;; This routine will return DE as a pointer in RAM to the pref list.
	;; Note that if you delete the appvar but keep the app installed, you
	;; will find that it is created again rather quickly.
	ld hl,appVarName
	rst rMov9ToOP1
	B_CALL _ChkFindSym
	jr c,LoadAppVar_Create
	ld a,b
	or a
	jr nz,LoadAppVar_Unarch
	ld (appVarSave),hl
	inc de
	inc de
	ld hl,appVarSig
	ld b,appVarSigLen
	call CompStrsN
	ex de,hl
	ld bc,appVarFDL
	add hl,bc
	ex de,hl
	ret z
	ld a,E_Duplicate	; DUPLICATE error; we simply don't know how to
	B_CALL _JError		; handle this situation

Mov66OPTemp:
	ld hl,OP1
	ld de,tempLoadingOPs
	jr Mov66
Mov66TempOP:
	ld hl,tempLoadingOPs
	ld de,OP1
Mov66:	call Mov11
	call Mov11
	call Mov11
	call Mov11
	call Mov11
	jp Mov11

appVarName:
	.db AppVarObj,"UnitMode"

;;; When modifying the appvar, make sure to update the AddPreference code,
;;; which deletes and reconstructs the entire appvar in the process of applying
;;; a preset preference list.

appVarDef:
appVarSig:
	.db "UnOp",0,8
appVarSigLen = $-appVarSig
appVarFD:
	;; Alternate hooks
ClearHooks:
appVarParser:
	.db 0,0,0
appVarHome:
	.db 0,0,0
appVarRawkey:
	.db 0,0,0
appVarDefPre:
	;; String-typing status
;  	.dw 0,0
	.db 0
appVarFixedLen = $-appVarDef
appVarFDL = $-appVarFD
appVarFixedEnd:
OFF_PARSER = appVarParser-appVarFixedEnd
OFF_HOMEHOOK = appVarHome-appVarFixedEnd
OFF_RAWKEY = appVarRawkey-appVarFixedEnd
	;; List of preferred units
	.dw 0,0
appVarDefLen = $-appVarDef
appVarDefPreLen = $-appVarDefPre

;;; Similar in theory to the above, but this is for rawkeyhook use
;;; - rather than trying to unarchive or create (often impossible for RKH)
;;; or throwing an unhelpful error, this one returns carry if the file
;;; isn't accessible.

LoadAppVar_RKH:
	call LoadAppVar_Quick
	ret z
	ld de,tempLoadingOPs
	call Mov11FrOP1
	ld hl,appVarName
	rst rMov9ToOP1
	B_CALL _ChkFindSym
	ld (appVarSave),hl
	ld a,b
	push af
	push de
	ld hl,tempLoadingOPs
	rst rMov9ToOP1
	pop de
	pop af
	ret c
	or a
	scf
	ret nz
	push hl
	inc de
	inc de
	ld hl,appVarSig
	ld b,appVarSigLen
	call CompStrsN
	pop hl
	ex de,hl
	ld bc,appVarFDL
	add hl,bc
	ex de,hl
	ret z
	scf
	ret

LoadAppVar_Quick:
	ld hl,(appVarSave)
	ld a,(hl)
	and $1f
	cp AppVarObj
	ret nz
	ld bc,-14
	add hl,bc
	ld de,AppVarVATName
	ld b,9
	call CompStrsN
	ret nz
	ld a,(hl)
	or a
	ret nz
	inc hl
	ld d,(hl)
	inc hl
	ld e,(hl)
	inc de
	inc de
	ld hl,appVarSig
	ld b,appVarSigLen
	call CompStrsN
	ret nz
	ld hl,appVarFDL
	add hl,de
	ex de,hl
	ret


;;; This is probably the most used system routine in this app besides
;;; OP-to-OP copy.

PopOP1:
	ld hl,(FPS)
	ld bc,-9
	add hl,bc
	ld a,(hl)
	ld de,OP1
	ld bc,9
	and $1f
	cp $0c
	jr nz,PopOP1_NotCplx
	sbc hl,bc
	ld (FPS),hl
	ldir
	inc de
	inc de
	ld bc,9
	ldir
	ret
PopOP1_NotCplx:
	ld (FPS),hl
	ldir
	ret



ParserGetFirst:
	;; Misnamed. Find the HL'th argument in the FPS
	push hl
	ex de,hl
	ld hl,(FPS)
	ld bc,-9
ParserGetFirst_Loop:
	ld a,d
	or e
	jr z,ParserGetFirst_Done
	dec de
	ld a,(hl)
	add hl,bc
	and $1f
	cp $0c
	jr nz,ParserGetFirst_Loop
	add hl,bc
	jr ParserGetFirst_Loop
ParserGetFirst_Done:
	push hl
	ld a,(hl)
	ld de,OP1
	ld bc,9
	and $1f
	cp $0c
	jr nz,ParserGetFirst_NotCplx
	sbc hl,bc
	ldir
	inc de
	inc de
	ld bc,9
ParserGetFirst_NotCplx:
	ldir
	pop de
	pop hl
	ret



CmpOP2OP4:
	ld hl,OP2+2
	ld de,OP4+2
	ld b,7
CompStrsN:
	ld a,(de)
	cp (hl)
	ret nz
	inc de
	inc hl
	djnz CompStrsN
	ret

CpHLDE:	ld a,h
	cp d
	ret nz
	ld a,l
	cp e
	ret

;;; Our errors: sadly, we are limited to the handful of unhelpful error
;;; messages TIOS provides.

ErrUnitMismatch:
	;; Trying to add numbers with incompatible units
	B_CALL _ErrDimMismatch

ErrDataType:
	;; Mixing an incompatible type (lists, matrices, actual
	;; complex numbers) along with a unit-number
	B_CALL _ErrDataType

ErrNonIntExpon:
	;; Result of calculation contains non-integer exponents
	B_CALL _ErrDimension	; (message is INVALID DIM)

ErrUnitOverflow:
	;; Exponent outside our range of -49 to +49
	B_CALL _ErrOverflow

ErrUnknownUnit:
	;; Unknown unit while parsing
ErrUndefined:
	;; Undefined variable while parsing
	B_CALL _ErrUndefined

ErrArchived:
	;; Apparently our appvar is archived
	ld a,E_Archived
	B_CALL _JError

ErrMemory:
	;; We're very low on RAM...
	B_CALL _ErrMemory

ErrArgument:
	;; A vague error that no one should be forced to try to debug.
	;; I think I only used this when emulating the behavior of TI
	;; functions.
	B_CALL _ErrArgument

ErrDomain:
	;; Same as above.
	B_CALL _ErrDomain

ErrSyntax:
	;; Generated while parsing
	B_CALL _ErrSyntax

PutSNL:	B_CALL _NewLine
PutS:	ld a,(hl)
	inc hl
	or a
	ret z
	cp $D6
	jr z,PutSNL
	B_CALL _PutC
	jr PutS

PutS_Upper:
	ld a,(hl)
	inc hl
	or a
	ret z
	cp 'a'
	jr c,PutS_Upper_OK
	sub 32
PutS_Upper_OK:
	B_CALL _PutC
	jr PutS_Upper

EraseNewLine:
	ld a,(curCol)
	or a
	ret z
EraseForceNewLine:
	ld a,' '
	B_CALL _PutC
	jr EraseNewLine

OP6ToOP2:
	ld hl,OP6
	jr Mov11OP2
OP8ToOP2:
	ld hl,OP8
	jr Mov11OP2
OP2ToOP10:
	ld de,OP10
	jr Mov11FrOP2
OP2ToOP8:
	ld de,OP8
	jr Mov11FrOP2
OP2ToOP7:
	ld de,OP7
	jr Mov11FrOP2
OP6ToOP1:
	ld hl,OP6
	jr Mov11OP1
OP7ToOP1:
	ld hl,OP7
	jr Mov11OP1
OP11ToOP1:
	ld hl,OP11
	jr Mov11OP1
OP1ToOP11:
	ld de,OP11
	jr Mov11FrOP1
OP1ToOP6:
	ld de,OP6
Mov11FrOP1:
	ld hl,OP1
	jr Mov11
OP11ToOP2:
	ld hl,OP11
	jr Mov11OP2
OP10ToOP2:
	ld hl,OP10
	jr Mov11OP2
OP3ToOP2:
	ld hl,OP3
	jr Mov11OP2
OP10ToOP1:
	ld hl,OP10
Mov11OP1:
	ld de,OP1
	jr Mov11
OP2ToOP4:
	ld de,OP4
Mov11FrOP2:
	ld hl,OP2
	jr Mov11
OP4ToOP2:
	ld hl,OP4
Mov11OP2:
	ld de,OP2
Mov11:	ldi
	ldi
Mov9:	ldi
Mov8:	ldi
Mov7:	ldi
Mov6:	ldi
 	ldi
	ldi
Mov3:	ldi
	ldi
	ldi
	ret
OP1ToOP9:
	ld de,OP9
	jr Mov11FrOP1
OP9ToOP1:
	ld hl,OP9
	jr Mov11OP1
OP12ToOP1011:
	ld de,OP10
Mov22FrOP1:
	ld hl,OP1
Mov22:
	call Mov11
	jr Mov11
OP1011ToOP12:
	ld hl,OP10
Mov22OP1:
	ld de,OP1
	call Mov11
	jr Mov11

OneThird:
	.db $00,$7f,$33,$33,$33,$33,$33,$33,$33,$33,$33
OneHalf:
	.db $00,$7f,$50,$00,$00,$00,$00,$00,$00,$00,$00

One:
	.db $00,$80,$10,$00,$00,$00,$00,$00
Three0s: .db $00,$00,$00
NoUnits:
	.db $8c,$80-97,$50,$50,$50,$50,$50,$50,$50


NullPrefixPtr:
	.dw NullPrefix


;;; The hooks, including the menu system
#include "hooks.asm"

;;; Math-related parser functions
#include "maths.asm"

;;; Text(, Disp_ and related code
#include "display.asm"

;;; Internal functions used to convert string->num
#include "read.asm"

;;; Internal functions used to convert num->string
#include "write.asm"


	.echo "Length of code: "
	.echo $-$4000
	.echo "\n"


MainMenuStrs:
	.db "Install",$D6,0
	.db "Uninstall",$D6,0
	.db "Advanced setup",0
	.db "Help",$D6,0
	.db "Quit",0

StartupErrorStr:
	.db "ERROR:",$D6
	.db "UnitMode appvar",$D6,0

StartupArcStr:
	.db "is archived. It",$D6
	.db "must be in RAM",$D6
	.db "for the app to",$D6
	.db "function.",0

StartupInvalStr:
	.db "seems to have",$D6
	.db "been created by",$D6
	.db "an older version"
	.db "or by another",$D6
	.db "app. It must be",$D6
	.db "removed.",0

AdvancedStrs:
ParserStr:
	.db "Parser",0
CxM_HomeStr:
	.db "Home",0
RawKeyStr:
 	.db "RawKey",0
	.db "Help",0
	.db "Main menu",0

OnStr:	.db "    On",0
OffStr:	.db "   Off",0
A_InUseStr:
	.db "In Use",0
SharedStr:
	.db "Shared",0
	
InstalledStr:
	.db "Installed",0

UninstalledStr:
	.db "Uninstalled",$D6,0

CancelStr:
	.db "Install aborted",0

InUseStr:
	.db " hook",$D6
	.db "is in use.",$D6
	.db "1:Overwrite",$D6
	.db "2:Share",$D6
	.db "3:Skip  4:Cancel",0

	.db 0
HelpText:
	.db "  HELP",$D6
	.db "Once installed,",$D6
	.db "this app allows",$D6
	.db "you to use units"
	.db "in the OS.",0

	.db "You can attach",$D6
	.db "units to any",$D6
	.db "number and",$D6
	.db "manipulate",$D6
	.db "normally.",0

	.db "Syntax:",$D6
	.db " 9.8(\"m\"/\"s\"",Lsquare,")",$D6
	.db " 3",Lexponent,"6*\"km\"*\"s\"",Linverse,$D6
	.db "Note: * or ()",$D6
	.db "are necessary.",0

	.db "Pressing STAT",$D6
	.db "twice will give",$D6
	.db "a menu with all",$D6
	.db "available units",$D6
	.db "and prefixes",0

	.db "Supports:",$D6
	.db "+ - * / ",Linverse,' ',Lsquare,' ',Lcube," ^",$D6
	.db Lroot,' ',LcubeR,Lroot,' ',LsupX,Lroot," = ",LNE," < ",LLE,$D6
	.db "> ",LGE,' ',Lconvert,"Frac ",Lconvert,"Dec",$D6
	.db Lconvert,"DMS Disp Text",0

	.db "Also, you can",$D6
	.db "use ^ to convert"
	.db $D6
	.db "(1*\"ft\")^\"m\"",$D6
	.db Lstore,"  .3048 m",0

	.db "Written by",$D6
	.db "Benjamin Moody",$D6
	.db $D6
	.db "benjamin @",$D6
	.db "  ecg.mit.edu",0,0

AHelpText:
	.db "  HOOK SETUP",$D6
	.db $D6
	.db "PARSER - needed",$D6
	.db "for all unit",$D6
	.db "functionality.",0

	.db "HOME - needed",$D6
	.db "to display units"
	.db "on homescreen",$D6
	.db "without using",$D6
	.db Lconvert,"Frac, etc.",0
	
	.db $D6
	.db "RAW KEY - needed"
	.db "for UNIT menu.",0,0

;;; Menu data
#include "menu.asm"

;;; Unit data
#include "unitdata.asm"


WelcomeStr:
	.db "Unit Operations"
 	.db "  Version "
	.db '0'+VMajor
	.db "."
	.db '0'+VMinor
	.db "  ",0


	.echo "Total length of app: "
	.echo $-$4000
	.echo "\n"

.end
END
