;;; -*- mode: TI-Asm; ti-asm-listing-file: "units.lst" -*-
;;;
;;;	Unit Operations Flash App for TI-83+
;;; 
;;;	Copyright 2003 Benjamin Moody
;;;
;;;  This program is free software; you can redistribute
;;;  and/or modify it under the terms of the GNU General
;;;  Public License as published by the Free Software
;;;  Foundation; either version 2 of the License, or (at
;;;  your option) any later version.
;;;
;;;  This program is distributed in the hope that it will be
;;;  useful, but WITHOUT ANY WARRANTY; without even the
;;;  implied warranty of MERCHANTABILITY or FITNESS FOR A
;;;  PARTICULAR PURPOSE. See the GNU General Public License
;;;  for more details.
;;;
;;;  You should have received a copy of the GNU General
;;;  Public License along with this program; if not, write
;;;  to the Free Software Foundation, Inc., 59 Temple Place
;;;  - Suite 330, Boston, MA 02111 USA.
;;;

;;; write.asm: ConvertToString and related routines

ConvertToString:
	;; Input: OP1-2 must be a unit number
	;; Output: temporary string variable, name in OP1.

	;; This routine uses OP1-OP4.

	call LoadAppVar
	ld (appVarPtr),de
	ld hl,formatString
	ld (hl),tSpace
	inc hl
	ld (formatPtr),hl

	;; If the number is zero, why bother?
	;; Avoid the ugliness of 'DIVIDE BY 0' ... and the dimension of zero
	;; doesn't matter anyway.
	ld a,(OP1+2)
	or a
	jr z,ConvertToString_Zero

	;; String format:
	;;  <Number> <Unit>*<Unit>^<Power>/<Unit>
	;; That is, the units are separated by tDotIcon, the positive from the
	;; negative by tDiv, and the units from the number by tSpace.

	;; We'll construct this string sort of inside-out.

	;; First, find all the units with positive exponents
	call ConvertToStringGetAllP
	call ConvertToStringExp

	ld hl,(formatPtr)
	dec hl
	ld (hl),tDiv

	;; Now find the units with negative exponents
	call UnitRecip
	call ConvertToStringGetAllP
	call ConvertToStringExp

;;; I'm sure there's some way to actually get FormBase et al to output tokens.
;;; This flag doesn't work as documented, though.
;  	set fmtEdit,(iy+fmtEditFlags)

	call RealOP1
	B_CALL _FPRecip

ConvertToString_Zero:

	;; Despite everything TI may try to tell you, FormBase takes A =
	;; maximum width, just as with FormReal.

	ld a,16
	B_CALL _FormBase
	push bc

	;; Since I don't know how to make FormBase do this, I need to convert
	;; '.' into tDecPt, '-' into tNeg, 'E' into tee, and '/' into tDiv.

	ld hl,OP3
	ld de,tempStringMem
	ld b,c
	;; Stuff to convert:
	;;  '.' -> tDecPt
	;;  '-' -> tNeg
	;;  'E' -> tEE
	;;  '/' -> tDiv
ConvertToStringFixLoop:
	ld a,(hl)
	ld c,tDecPt
	cp '.'
	jr z,ConvertToStringFixChar
	ld c,tChs
	cp Lneg
	jr z,ConvertToStringFixChar
	ld c,tee
	cp Lexponent
	jr z,ConvertToStringFixChar
	ld c,tDiv
	cp '/'
	jr nz,ConvertToStringFixNotChar
ConvertToStringFixChar:
	ld a,c
ConvertToStringFixNotChar:
	ld (de),a
	inc hl
	inc de
	djnz ConvertToStringFixLoop

	;; Now create a temporary string variable to return output
	ld l,StrngObj
 	call NewTempName
; 	ld a,StrngObj
; 	ld (OP1),a
; 	ld a,$24
; 	ld (OP1+1),a
; 	ld hl,(pTempCnt)
; 	ld (OP1+2),hl
; 	xor a
; 	ld (OP1+4),a

	pop hl
	push hl
	ld de,(formatPtr)
	add hl,de
	ld de,(-formatString)-1
	add hl,de
	B_CALL _CreateStrng
	inc de
	inc de
	ld hl,tempStringMem
	pop bc
	ldir
	ld hl,(formatPtr)
	ld bc,(-formatString)-1
	add hl,bc
	ld a,l
	or h
	jr z,ConvertToStringNull
	ld c,l
	ld b,h
	ld hl,formatString
	ldir
ConvertToStringNull:
	ld l,StrngObj
LastTempName:
	ld h,$24
	ld (OP1),hl
	ld hl,(pTempCnt)
	dec hl
	ld (OP1+2),hl
	xor a
	ld (OP1+4),a

	ret

NewTempName:
	ld h,$24
	ld (OP1),hl
	ld hl,(pTempCnt)
	ld (OP1+2),hl
	inc hl
	ld (pTempCnt),hl
	xor a
	ld (OP1+4),a
	ret

ConvertToStringMore:
 	ld a,(currentExponent)
 	inc a
 	ld (currentExponent),a
	jr ConvertToStringNoDisplay

ConvertToStringGetAllP:
	ld hl,0
	ld (tempUnitPtr),hl
	ld a,1
	ld (currentExponent),a
ConvertToStringGetAllPLoop:
	call FindPreferredUnit
	ret nc

	;; Check: have we had any yet?
	push hl
	or a
	ld hl,(tempUnitPtr)
	sbc hl,de
	pop hl
	ld (tempUnitPtr),de

	;; If we already have some, increment the exponent
 	jr z,ConvertToStringMore

 	;; If not, add the exponent if it needs to be displayed,
 	;; and display the new string
	push de
 	call ConvertToStringExp
	pop de

	;; DE->unit string, HL->->prefix
	push de
	;; Get pointer to prefix
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl		; HL is now the address of the prefix
	ld de,(formatPtr)
	ld a,(hl)
	or a
	jr z,ConvertToStringNoPrefix
	ld c,a
	ld b,0
	inc hl
	ldir
	;; Get the exponent of the prefix
	ld a,(hl)
ConvertToStringNoPrefix:
	ld (unitPrefix),a
	;; Now add the string for the unit
	pop hl
	ld c,(hl)
	ld b,0
	inc hl
	ldir
	ld a,tDotIcon
	ld (de),a		; Make it look cool, at least
	inc de
	ld (formatPtr),de
	;; Save the address of the unit
	ld (lastPrefixPtr),hl

ConvertToStringNoDisplay:
	;; Divide by prefix
	ld a,(unitPrefix)
	ld b,a
	ld a,(OP1+1)
	sub b
	ld (OP1+1),a
	;; Reduce the units in OP2
	ld hl,(lastPrefixPtr)
	push hl
	ld bc,8
	add hl,bc
	ld bc,7
	ld de,OP4+2
	ldir
	call SubUnitsOP2OP4
	pop hl
	;; Now we want to divide by the constant
	ld de,OP2+1
	ld bc,8
	ldir
	xor a
	ld (OP2),a
	B_CALL _FPDiv
	;; and return the new units to OP2.
	call OP4ToOP2

	;; We still want to check for more than one of a unit, but I'm
	;; too lazy at the moment.

	jr ConvertToStringGetAllPLoop

ConvertToStringExp:
	ld a,(currentExponent)
	dec a
	ret z
	dec a
	ld b,tSqr
	jr z,ConvertToStringExpSimple
	dec a
	ld b,tCube
	jr z,ConvertToStringExpSimple
	add a,3
	ld b,a
	ld de,(formatPtr)
	dec de
	ld a,tPower
	ld (de),a
	inc de
	ld a,b
	call PrintInt
	ld a,tDotIcon
	ld (de),a
	inc de
	ld (formatPtr),de
	ld a,1
	ld (currentExponent),a
	ret
ConvertToStringExpSimple:
	ld de,(formatPtr)
	dec de
	ld a,b
	ld (de),a
	inc de
	ld a,tDotIcon
	ld (de),a
	inc de
	ld (formatPtr),de
	ld a,1
	ld (currentExponent),a
	ret

FindPreferredUnit:
	;; Input: OP2 is units. Find one appropriate
	;; Carry flag on success, and HL->ptr->prefix and DE->unit string
	ld hl,(appVarPtr)	; HL = start of preferred units list
FindPreferredUnitLoop:
	;; Each entry in the preferred units list consists of unit-ptr,
	;; followed by prefix-ptr.
	push hl
	 ld hl,OP2+2
	 ld a,$50
	 ld b,7
FindPreferredUnitLoop0:
	 cp (hl)
	 jr nz,FindPreferredUnitNotZ
	 inc hl
	 djnz FindPreferredUnitLoop0
	 or a
	 pop hl
	ret
FindPreferredUnitNotZ:
	 pop hl
	ld e,(hl)
	inc hl
	ld d,(hl)		; Get unit
	inc hl
	ld a,d
	or e
	jr z,FindPreferredUnitNotInList
;	call UnitLookupDE
	push hl			; Save our location in the list
	 push de
	  ld a,(de)
	  ld l,a
	  ld h,0
	  add hl,de		; Skip the token string,
	  ld de,9			; including the length byte,
	  add hl,de		; and the constant factor
	  call CheckOP2Superunit
	  pop de
	 pop hl
	ret c			; If it returned carry, the unit was valid
	inc hl
	inc hl			; Move to next preference
	jr FindPreferredUnitLoop
FindPreferredUnitNotInList:
	ld hl,UnitStrings_WRITE
FindPreferredUnitDefaultLoop:
	ld a,(hl)
	or a
	ret z
	push hl
	add a,9
	ld e,a
	add hl,de
	call CheckOP2Superunit
	pop de
	ld hl,NullPrefixPtr
	ret c
	ex de,hl
	ld d,0
	ld a,(hl)
	add a,16
	ld e,a
	add hl,de
	jr FindPreferredUnitDefaultLoop


CheckOP2Superunit:
	;; We want to check if the unit is worth adding.
	;; For each basic unit,
	;;   BU(Num) > 0 and BU(Num) >= BU(Pref)
	;;    OR
	;;   BU(Num) < 0 and BU(Num) <= BU(Pref)
	;;    OR
	;;   BU(Pref) = 0
	;; Return C if it matches.
	ex de,hl
	ld b,7
	ld hl,OP2+2
CheckOP2Superunit_Loop:
	ld a,(de)
	cp $50
	jr z,CheckOP2Superunit_OK
	rl c
	cp (hl)
	jr z,CheckOP2Superunit_OK
	;; Comparison 1 returns carry if num > pref
	;; Comparison 2 returns carry if 0 > pref
	rla
	add a,c
	and 1
	;; If exactly one carried, then |num| > |pref| and we're OK
	ret z
CheckOP2Superunit_OK:
	inc de
	inc hl
	djnz CheckOP2Superunit_Loop
	scf
	ret


FillString:
	;; For homescreen display: pad a string with tSpace up to 16 chars
	B_CALL _ChkFindSym
	ret c
	ld a,b
	or a
	ret nz
	ld a,(de)
	ld b,a
	inc de
	ld a,(de)
	or a
	ret nz
	inc de
	push de
	 ex de,hl
	 ld c,a
FillStringLengthLoop:
	 push hl
	  push bc
;;; GetTokLen is inconsistent. The best explanation I can come up with is that
;;; it is inconsistent between base versions. I NEED a different basecall for
;;; this. Get_Tok_String used to be undocumented, but it might work...
	   B_CALL _Get_Tok_Strng
	   pop bc
	  pop hl
	 add a,c
	 jp c,PopRet		; I don't think this situation is possible,
	 ld c,a			; but it's better to be safe
	 ld a,(hl)
	 inc hl
	 B_CALL _IsA2ByteTok
	 jr nz,FillStringLength1B
	 inc hl
	 dec b
FillStringLength1B:
	 djnz FillStringLengthLoop

	;; C is the number of characters

	 pop hl			; start address

	ld a,c
	sub 16
	ret nc
	neg
	ld b,a

;	push hl
;	push bc
;	ld l,c
;	ld h,b
;	B_CALL _DispHL
;	pop bc
;	pop hl

	push bc			; number of chars to add
	 push hl
	  ld l,b
	  ld h,0
	  B_CALL _EnoughMem	; This may seem dumb, but InsertMem doesn't
	  jp c,Pop2Ret		; actually check for available memory.
	  ex de,hl
	  pop de
	 B_CALL _InsertMem
	 pop bc
	dec de
	dec de
	ld a,(de)
	add a,b
	ld (de),a
	inc de
	inc de
FillStringLoop:
	ld a,tSpace
	ld (de),a
	inc de
	djnz FillStringLoop
	ret

PrintInt:
	;; Put an 8-bit number into a string at DE
	cp 100
	jr c,PrintInt_2
	ld b,'1'
	sub 200
	jr c,PrintInt_3
	ld b,'2'
	sub 100
PrintInt_3:
	add a,100
	ld c,a
	ld a,b
	ld (de),a
	inc de
	ld a,c
PrintInt_2:
	cp 10
	jr c,PrintInt_1
	push de
	ld d,a
	ld e,10
	call DDivE
	ld c,a
	ld a,d
	add a,'0'
	pop de
	ld (de),a
	inc de
	ld a,c
PrintInt_1:
	add a,'0'
	ld (de),a
	inc de
	ret
